"""
Amplify resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_amplify_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect Amplify resources: apps, branches.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    amplify = session.client('amplify', region_name=region)

    # Amplify Apps
    try:
        paginator = amplify.get_paginator('list_apps')
        for page in paginator.paginate():
            for app in page.get('apps', []):
                app_id = app['appId']
                app_arn = app['appArn']
                app_name = app['name']

                # Tags are included in the response
                tags = app.get('tags', {})

                resources.append({
                    'service': 'amplify',
                    'type': 'app',
                    'id': app_id,
                    'arn': app_arn,
                    'name': app_name,
                    'region': region,
                    'details': {
                        'description': app.get('description'),
                        'repository': app.get('repository'),
                        'platform': app.get('platform'),
                        'create_time': str(app.get('createTime', '')),
                        'update_time': str(app.get('updateTime', '')),
                        'default_domain': app.get('defaultDomain'),
                        'enable_branch_auto_build': app.get('enableBranchAutoBuild'),
                        'enable_branch_auto_deletion': app.get('enableBranchAutoDeletion'),
                        'enable_basic_auth': app.get('enableBasicAuth'),
                        'production_branch': app.get('productionBranch', {}).get('branchName'),
                        'custom_rules': len(app.get('customRules', [])),
                        'environment_variables': len(app.get('environmentVariables', {})),
                        'iam_service_role_arn': app.get('iamServiceRoleArn'),
                    },
                    'tags': tags
                })

                # Branches for this app
                try:
                    branch_paginator = amplify.get_paginator('list_branches')
                    for branch_page in branch_paginator.paginate(appId=app_id):
                        for branch in branch_page.get('branches', []):
                            branch_name = branch['branchName']
                            branch_arn = branch['branchArn']

                            branch_tags = branch.get('tags', {})

                            resources.append({
                                'service': 'amplify',
                                'type': 'branch',
                                'id': f"{app_id}/{branch_name}",
                                'arn': branch_arn,
                                'name': branch_name,
                                'region': region,
                                'details': {
                                    'app_id': app_id,
                                    'app_name': app_name,
                                    'description': branch.get('description'),
                                    'stage': branch.get('stage'),
                                    'display_name': branch.get('displayName'),
                                    'enable_notification': branch.get('enableNotification'),
                                    'create_time': str(branch.get('createTime', '')),
                                    'update_time': str(branch.get('updateTime', '')),
                                    'enable_auto_build': branch.get('enableAutoBuild'),
                                    'total_number_of_jobs': branch.get('totalNumberOfJobs'),
                                    'enable_basic_auth': branch.get('enableBasicAuth'),
                                    'active_job_id': branch.get('activeJobId'),
                                    'ttl': branch.get('ttl'),
                                    'enable_pull_request_preview': branch.get('enablePullRequestPreview'),
                                    'pull_request_environment_name': branch.get('pullRequestEnvironmentName'),
                                    'backend_environment_arn': branch.get('backendEnvironmentArn'),
                                },
                                'tags': branch_tags
                            })
                except Exception:
                    pass

                # Domain Associations for this app
                try:
                    domain_paginator = amplify.get_paginator('list_domain_associations')
                    for domain_page in domain_paginator.paginate(appId=app_id):
                        for domain in domain_page.get('domainAssociations', []):
                            domain_name = domain['domainName']
                            domain_arn = domain['domainAssociationArn']

                            resources.append({
                                'service': 'amplify',
                                'type': 'domain',
                                'id': f"{app_id}/{domain_name}",
                                'arn': domain_arn,
                                'name': domain_name,
                                'region': region,
                                'details': {
                                    'app_id': app_id,
                                    'app_name': app_name,
                                    'domain_status': domain.get('domainStatus'),
                                    'status_reason': domain.get('statusReason'),
                                    'enable_auto_sub_domain': domain.get('enableAutoSubDomain'),
                                    'auto_sub_domain_creation_patterns': domain.get('autoSubDomainCreationPatterns', []),
                                    'sub_domains': [sd.get('subDomainSetting', {}).get('branchName') for sd in domain.get('subDomains', [])],
                                    'certificate_verification_dns_record': domain.get('certificateVerificationDNSRecord'),
                                },
                                'tags': {}
                            })
                except Exception:
                    pass
    except Exception:
        pass

    return resources



"""
AWS Resource Explorer resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


def collect_resourceexplorer_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Resource Explorer resources: indexes and views.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    explorer = session.client('resource-explorer-2', region_name=region)

    # Indexes
    try:
        paginator = explorer.get_paginator('list_indexes')
        for page in paginator.paginate():
            for index in page.get('Indexes', []):
                index_arn = index['Arn']
                index_region = index.get('Region', region)
                index_type = index.get('Type', 'LOCAL')

                # Only collect indexes for the current region to avoid duplicates
                if index_region != region:
                    continue

                details = {
                    'type': index_type,
                }

                resources.append({
                    'service': 'resource-explorer-2',
                    'type': 'index',
                    'id': index_arn.split('/')[-1] if '/' in index_arn else index_arn,
                    'arn': index_arn,
                    'name': f"{index_type.lower()}-index",
                    'region': index_region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Views
    try:
        paginator = explorer.get_paginator('list_views')
        for page in paginator.paginate():
            for view_arn in page.get('Views', []):
                # Extract view name from ARN
                view_name = view_arn.split('/view/')[-1].split('/')[0] if '/view/' in view_arn else view_arn

                resources.append({
                    'service': 'resource-explorer-2',
                    'type': 'view',
                    'id': view_name,
                    'arn': view_arn,
                    'name': view_name,
                    'region': region,
                    'details': {},
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
AWS Elemental MediaLive resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


# MediaLive supported regions (from https://docs.aws.amazon.com/general/latest/gr/medialive_region.html)
MEDIALIVE_REGIONS = {
    'us-east-1', 'us-east-2', 'us-west-2',
    'ap-south-1', 'ap-south-2', 'ap-northeast-1', 'ap-northeast-2', 'ap-northeast-3',
    'ap-southeast-1', 'ap-southeast-2', 'ap-southeast-4',
    'ca-central-1',
    'eu-central-1', 'eu-west-1', 'eu-west-2', 'eu-west-3', 'eu-north-1',
    'me-central-1',
    'sa-east-1',
}


def collect_medialive_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS MediaLive resources: channels, inputs, and input security groups.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    # Skip unsupported regions
    if region not in MEDIALIVE_REGIONS:
        return []

    resources = []
    ml = session.client('medialive', region_name=region)

    # Channels
    try:
        paginator = ml.get_paginator('list_channels')
        for page in paginator.paginate():
            for channel in page.get('Channels', []):
                channel_id = channel['Id']
                channel_arn = channel.get('Arn', '')
                channel_name = channel.get('Name', channel_id)

                details = {
                    'state': channel.get('State'),
                    'channel_class': channel.get('ChannelClass'),
                    'log_level': channel.get('LogLevel'),
                    'role_arn': channel.get('RoleArn'),
                    'pipelines_running_count': channel.get('PipelinesRunningCount'),
                    'input_attachments_count': len(channel.get('InputAttachments', [])),
                    'destinations_count': len(channel.get('Destinations', [])),
                }

                maintenance = channel.get('Maintenance', {})
                if maintenance:
                    details['maintenance_day'] = maintenance.get('MaintenanceDay')
                    details['maintenance_start_time'] = maintenance.get('MaintenanceStartTime')

                tags = channel.get('Tags', {})

                resources.append({
                    'service': 'medialive',
                    'type': 'channel',
                    'id': channel_id,
                    'arn': channel_arn,
                    'name': channel_name,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Inputs
    try:
        paginator = ml.get_paginator('list_inputs')
        for page in paginator.paginate():
            for input_item in page.get('Inputs', []):
                input_id = input_item['Id']
                input_arn = input_item.get('Arn', '')
                input_name = input_item.get('Name', input_id)

                details = {
                    'state': input_item.get('State'),
                    'type': input_item.get('Type'),
                    'input_class': input_item.get('InputClass'),
                    'input_source_type': input_item.get('InputSourceType'),
                    'attached_channels': input_item.get('AttachedChannels', []),
                    'security_groups': input_item.get('SecurityGroups', []),
                    'sources_count': len(input_item.get('Sources', [])),
                    'destinations_count': len(input_item.get('Destinations', [])),
                    'role_arn': input_item.get('RoleArn'),
                }

                tags = input_item.get('Tags', {})

                resources.append({
                    'service': 'medialive',
                    'type': 'input',
                    'id': input_id,
                    'arn': input_arn,
                    'name': input_name,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Input Security Groups
    try:
        paginator = ml.get_paginator('list_input_security_groups')
        for page in paginator.paginate():
            for sg in page.get('InputSecurityGroups', []):
                sg_id = sg['Id']
                sg_arn = sg.get('Arn', '')

                details = {
                    'state': sg.get('State'),
                    'whitelist_rules_count': len(sg.get('WhitelistRules', [])),
                    'inputs': sg.get('Inputs', []),
                }

                tags = sg.get('Tags', {})

                resources.append({
                    'service': 'medialive',
                    'type': 'input-security-group',
                    'id': sg_id,
                    'arn': sg_arn,
                    'name': sg_id,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    return resources



"""
CloudFormation resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_cloudformation_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect CloudFormation resources: stacks, stack sets.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    cfn = session.client('cloudformation', region_name=region)

    # CloudFormation Stacks
    try:
        paginator = cfn.get_paginator('describe_stacks')
        for page in paginator.paginate():
            for stack in page.get('Stacks', []):
                stack_name = stack['StackName']
                stack_id = stack['StackId']

                # Tags are included in the response
                tags = {}
                for tag in stack.get('Tags', []):
                    tags[tag.get('Key', '')] = tag.get('Value', '')

                resources.append({
                    'service': 'cloudformation',
                    'type': 'stack',
                    'id': stack_id,
                    'arn': stack_id,  # StackId is actually the ARN
                    'name': stack_name,
                    'region': region,
                    'details': {
                        'status': stack.get('StackStatus'),
                        'status_reason': stack.get('StackStatusReason'),
                        'description': stack.get('Description'),
                        'creation_time': str(stack.get('CreationTime', '')),
                        'last_updated_time': str(stack.get('LastUpdatedTime', '')),
                        'deletion_time': str(stack.get('DeletionTime', '')) if stack.get('DeletionTime') else None,
                        'enable_termination_protection': stack.get('EnableTerminationProtection'),
                        'drift_status': stack.get('DriftInformation', {}).get('StackDriftStatus'),
                        'root_id': stack.get('RootId'),
                        'parent_id': stack.get('ParentId'),
                        'role_arn': stack.get('RoleARN'),
                        'outputs_count': len(stack.get('Outputs', [])),
                        'parameters_count': len(stack.get('Parameters', [])),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # CloudFormation Stack Sets
    try:
        paginator = cfn.get_paginator('list_stack_sets')
        for page in paginator.paginate(Status='ACTIVE'):
            for ss in page.get('Summaries', []):
                ss_name = ss['StackSetName']
                ss_id = ss['StackSetId']

                try:
                    # Get stack set details
                    ss_response = cfn.describe_stack_set(StackSetName=ss_name)
                    ss_detail = ss_response.get('StackSet', {})

                    # Tags are included in detail response
                    tags = {}
                    for tag in ss_detail.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')

                    resources.append({
                        'service': 'cloudformation',
                        'type': 'stack-set',
                        'id': ss_id,
                        'arn': ss_detail.get('StackSetARN', f"arn:aws:cloudformation:{region}:{account_id}:stackset/{ss_name}:{ss_id}"),
                        'name': ss_name,
                        'region': region,
                        'details': {
                            'status': ss_detail.get('Status'),
                            'description': ss_detail.get('Description'),
                            'permission_model': ss_detail.get('PermissionModel'),
                            'auto_deployment': ss_detail.get('AutoDeployment'),
                            'organizational_unit_ids': ss_detail.get('OrganizationalUnitIds', []),
                            'administration_role_arn': ss_detail.get('AdministrationRoleARN'),
                            'execution_role_name': ss_detail.get('ExecutionRoleName'),
                            'drift_status': ss_detail.get('StackSetDriftDetectionDetails', {}).get('DriftStatus'),
                            'managed_execution': ss_detail.get('ManagedExecution', {}).get('Active'),
                        },
                        'tags': tags
                    })
                except Exception:
                    pass
    except Exception:
        pass

    return resources



"""
IAM Identity Center (SSO) resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_sso_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect IAM Identity Center resources: instances, permission sets, users, groups.

    Note: IAM Identity Center is regional but only ONE instance can be active
    per AWS Organization. This collector only runs in the region where the
    instance is deployed.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    sso_admin = session.client('sso-admin', region_name=region)

    # List Identity Center instances
    instances = []
    try:
        paginator = sso_admin.get_paginator('list_instances')
        for page in paginator.paginate():
            instances.extend(page.get('Instances', []))
    except Exception:
        pass

    for instance in instances:
        instance_arn = instance['InstanceArn']
        identity_store_id = instance.get('IdentityStoreId')

        # Get instance details
        resources.append({
            'service': 'sso',
            'type': 'instance',
            'id': instance_arn.split('/')[-1],
            'arn': instance_arn,
            'name': instance.get('Name') or f"sso-instance-{instance_arn.split('/')[-1][:8]}",
            'region': region,
            'details': {
                'identity_store_id': identity_store_id,
                'owner_account_id': instance.get('OwnerAccountId'),
                'status': instance.get('Status'),
                'created_date': str(instance.get('CreatedDate', '')),
            },
            'tags': {}
        })

        # Permission Sets for this instance
        try:
            ps_paginator = sso_admin.get_paginator('list_permission_sets')
            for ps_page in ps_paginator.paginate(InstanceArn=instance_arn):
                for ps_arn in ps_page.get('PermissionSets', []):
                    try:
                        ps_response = sso_admin.describe_permission_set(
                            InstanceArn=instance_arn,
                            PermissionSetArn=ps_arn
                        )
                        ps = ps_response.get('PermissionSet', {})

                        # Get tags
                        tags = {}
                        try:
                            tag_response = sso_admin.list_tags_for_resource(
                                InstanceArn=instance_arn,
                                ResourceArn=ps_arn
                            )
                            for tag in tag_response.get('Tags', []):
                                tags[tag.get('Key', '')] = tag.get('Value', '')
                        except Exception:
                            pass

                        resources.append({
                            'service': 'sso',
                            'type': 'permission-set',
                            'id': ps_arn.split('/')[-1],
                            'arn': ps_arn,
                            'name': ps.get('Name', ps_arn.split('/')[-1]),
                            'region': region,
                            'details': {
                                'instance_arn': instance_arn,
                                'description': ps.get('Description'),
                                'session_duration': ps.get('SessionDuration'),
                                'relay_state': ps.get('RelayState'),
                                'created_date': str(ps.get('CreatedDate', '')),
                            },
                            'tags': tags
                        })
                    except Exception:
                        pass
        except Exception:
            pass

        # Users and Groups from Identity Store
        if identity_store_id:
            identitystore = session.client('identitystore', region_name=region)

            # Users
            try:
                user_paginator = identitystore.get_paginator('list_users')
                for user_page in user_paginator.paginate(IdentityStoreId=identity_store_id):
                    for user in user_page.get('Users', []):
                        user_id = user['UserId']
                        user_name = user.get('UserName', user_id)

                        # Build display name
                        display_name = user.get('DisplayName')
                        if not display_name:
                            name_obj = user.get('Name', {})
                            if name_obj:
                                display_name = f"{name_obj.get('GivenName', '')} {name_obj.get('FamilyName', '')}".strip()

                        # Get primary email
                        emails = user.get('Emails', [])
                        primary_email = None
                        for email in emails:
                            if email.get('Primary'):
                                primary_email = email.get('Value')
                                break
                        if not primary_email and emails:
                            primary_email = emails[0].get('Value')

                        resources.append({
                            'service': 'sso',
                            'type': 'user',
                            'id': user_id,
                            'arn': f"arn:aws:identitystore::{account_id}:identitystore/{identity_store_id}/user/{user_id}",
                            'name': user_name,
                            'region': region,
                            'details': {
                                'identity_store_id': identity_store_id,
                                'display_name': display_name,
                                'email': primary_email,
                                'external_id': user.get('ExternalIds', [{}])[0].get('Id') if user.get('ExternalIds') else None,
                                'identity_provider': user.get('ExternalIds', [{}])[0].get('Issuer') if user.get('ExternalIds') else None,
                            },
                            'tags': {}
                        })
            except Exception:
                pass

            # Groups
            try:
                group_paginator = identitystore.get_paginator('list_groups')
                for group_page in group_paginator.paginate(IdentityStoreId=identity_store_id):
                    for group in group_page.get('Groups', []):
                        group_id = group['GroupId']
                        group_name = group.get('DisplayName', group_id)

                        resources.append({
                            'service': 'sso',
                            'type': 'group',
                            'id': group_id,
                            'arn': f"arn:aws:identitystore::{account_id}:identitystore/{identity_store_id}/group/{group_id}",
                            'name': group_name,
                            'region': region,
                            'details': {
                                'identity_store_id': identity_store_id,
                                'description': group.get('Description'),
                                'external_id': group.get('ExternalIds', [{}])[0].get('Id') if group.get('ExternalIds') else None,
                                'identity_provider': group.get('ExternalIds', [{}])[0].get('Issuer') if group.get('ExternalIds') else None,
                            },
                            'tags': {}
                        })
            except Exception:
                pass

    return resources



"""
Amazon EventBridge Schema Registry resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_schemas_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect EventBridge Schema Registry resources: registries, schemas, discoverers.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    schemas = session.client('schemas', region_name=region)

    # Registries (skip AWS-owned registries)
    try:
        paginator = schemas.get_paginator('list_registries')
        for page in paginator.paginate():
            for registry in page.get('Registries', []):
                registry_arn = registry.get('RegistryArn', '')
                registry_name = registry.get('RegistryName', registry_arn.split('/')[-1])

                # Skip AWS-owned registries
                if registry_name.startswith('aws.'):
                    continue

                details = {}

                # Get tags
                tags = {}
                try:
                    tag_response = schemas.list_tags_for_resource(ResourceArn=registry_arn)
                    tags = tag_response.get('Tags', {})
                except Exception:
                    pass

                resources.append({
                    'service': 'schemas',
                    'type': 'registry',
                    'id': registry_name,
                    'arn': registry_arn,
                    'name': registry_name,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Discoverers
    try:
        paginator = schemas.get_paginator('list_discoverers')
        for page in paginator.paginate():
            for discoverer in page.get('Discoverers', []):
                discoverer_id = discoverer.get('DiscovererId', '')
                discoverer_arn = discoverer.get('DiscovererArn', '')

                details = {
                    'source_arn': discoverer.get('SourceArn'),
                    'state': discoverer.get('State'),
                    'cross_account': discoverer.get('CrossAccount'),
                }

                # Get tags
                tags = {}
                try:
                    tag_response = schemas.list_tags_for_resource(ResourceArn=discoverer_arn)
                    tags = tag_response.get('Tags', {})
                except Exception:
                    pass

                resources.append({
                    'service': 'schemas',
                    'type': 'discoverer',
                    'id': discoverer_id,
                    'arn': discoverer_arn,
                    'name': discoverer_id,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    return resources



"""
Aurora DSQL resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_dsql_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect Aurora DSQL resources: clusters.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    dsql = session.client('dsql', region_name=region)

    # DSQL Clusters
    try:
        paginator = dsql.get_paginator('list_clusters')
        for page in paginator.paginate():
            for cluster_summary in page.get('clusters', []):
                cluster_id = cluster_summary['identifier']

                try:
                    # Get cluster details
                    cluster = dsql.get_cluster(identifier=cluster_id)

                    cluster_arn = cluster.get('arn', f"arn:aws:dsql:{region}:{account_id}:cluster/{cluster_id}")

                    # Tags are included in get_cluster response
                    tags = cluster.get('tags', {})

                    encryption = cluster.get('encryptionDetails', {})
                    multi_region = cluster.get('multiRegionProperties', {})

                    resources.append({
                        'service': 'dsql',
                        'type': 'cluster',
                        'id': cluster_id,
                        'arn': cluster_arn,
                        'name': cluster_id,
                        'region': region,
                        'details': {
                            'status': cluster.get('status'),
                            'endpoint': cluster.get('endpoint'),
                            'creation_time': str(cluster.get('creationTime', '')),
                            'deletion_protection_enabled': cluster.get('deletionProtectionEnabled'),
                            'encryption_type': encryption.get('encryptionType'),
                            'encryption_status': encryption.get('encryptionStatus'),
                            'kms_key_arn': encryption.get('kmsKeyArn'),
                            'witness_region': multi_region.get('witnessRegion'),
                            'linked_clusters': multi_region.get('clusters', []),
                        },
                        'tags': tags
                    })
                except Exception:
                    pass
    except Exception:
        pass

    return resources



"""
AWS Network Manager resource collector.

Note: Network Manager is a global service.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


def collect_networkmanager_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Network Manager resources: global networks, sites, devices, links, and connections.

    Note: Network Manager is a global service, called from us-west-2.

    Args:
        session: boto3.Session to use
        region: AWS region (ignored - global service)
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    nm = session.client('networkmanager', region_name='us-west-2')

    # Global Networks
    try:
        paginator = nm.get_paginator('describe_global_networks')
        for page in paginator.paginate():
            for network in page.get('GlobalNetworks', []):
                network_id = network['GlobalNetworkId']
                network_arn = network.get('GlobalNetworkArn', '')

                details = {
                    'description': network.get('Description'),
                    'state': network.get('State'),
                    'created_at': str(network.get('CreatedAt', '')),
                }

                tags = {t['Key']: t['Value'] for t in network.get('Tags', [])}

                resources.append({
                    'service': 'networkmanager',
                    'type': 'global-network',
                    'id': network_id,
                    'arn': network_arn,
                    'name': network.get('Description', network_id),
                    'region': 'global',
                    'details': details,
                    'tags': tags
                })

                # Sites for this global network
                try:
                    site_paginator = nm.get_paginator('get_sites')
                    for site_page in site_paginator.paginate(GlobalNetworkId=network_id):
                        for site in site_page.get('Sites', []):
                            site_id = site['SiteId']
                            site_arn = site.get('SiteArn', '')

                            site_details = {
                                'global_network_id': network_id,
                                'description': site.get('Description'),
                                'state': site.get('State'),
                                'created_at': str(site.get('CreatedAt', '')),
                            }

                            location = site.get('Location', {})
                            if location:
                                site_details['address'] = location.get('Address')
                                site_details['latitude'] = location.get('Latitude')
                                site_details['longitude'] = location.get('Longitude')

                            site_tags = {t['Key']: t['Value'] for t in site.get('Tags', [])}

                            resources.append({
                                'service': 'networkmanager',
                                'type': 'site',
                                'id': site_id,
                                'arn': site_arn,
                                'name': site.get('Description', site_id),
                                'region': 'global',
                                'details': site_details,
                                'tags': site_tags
                            })
                except Exception:
                    pass

                # Devices for this global network
                try:
                    device_paginator = nm.get_paginator('get_devices')
                    for device_page in device_paginator.paginate(GlobalNetworkId=network_id):
                        for device in device_page.get('Devices', []):
                            device_id = device['DeviceId']
                            device_arn = device.get('DeviceArn', '')

                            device_details = {
                                'global_network_id': network_id,
                                'site_id': device.get('SiteId'),
                                'description': device.get('Description'),
                                'type': device.get('Type'),
                                'vendor': device.get('Vendor'),
                                'model': device.get('Model'),
                                'serial_number': device.get('SerialNumber'),
                                'state': device.get('State'),
                                'created_at': str(device.get('CreatedAt', '')),
                            }

                            device_tags = {t['Key']: t['Value'] for t in device.get('Tags', [])}

                            resources.append({
                                'service': 'networkmanager',
                                'type': 'device',
                                'id': device_id,
                                'arn': device_arn,
                                'name': device.get('Description', device_id),
                                'region': 'global',
                                'details': device_details,
                                'tags': device_tags
                            })
                except Exception:
                    pass

                # Links for this global network
                try:
                    link_paginator = nm.get_paginator('get_links')
                    for link_page in link_paginator.paginate(GlobalNetworkId=network_id):
                        for link in link_page.get('Links', []):
                            link_id = link['LinkId']
                            link_arn = link.get('LinkArn', '')

                            link_details = {
                                'global_network_id': network_id,
                                'site_id': link.get('SiteId'),
                                'description': link.get('Description'),
                                'type': link.get('Type'),
                                'provider': link.get('Provider'),
                                'state': link.get('State'),
                                'created_at': str(link.get('CreatedAt', '')),
                            }

                            bandwidth = link.get('Bandwidth', {})
                            if bandwidth:
                                link_details['upload_speed_mbps'] = bandwidth.get('UploadSpeed')
                                link_details['download_speed_mbps'] = bandwidth.get('DownloadSpeed')

                            link_tags = {t['Key']: t['Value'] for t in link.get('Tags', [])}

                            resources.append({
                                'service': 'networkmanager',
                                'type': 'link',
                                'id': link_id,
                                'arn': link_arn,
                                'name': link.get('Description', link_id),
                                'region': 'global',
                                'details': link_details,
                                'tags': link_tags
                            })
                except Exception:
                    pass

                # Connections for this global network
                try:
                    conn_paginator = nm.get_paginator('get_connections')
                    for conn_page in conn_paginator.paginate(GlobalNetworkId=network_id):
                        for conn in conn_page.get('Connections', []):
                            conn_id = conn['ConnectionId']
                            conn_arn = conn.get('ConnectionArn', '')

                            conn_details = {
                                'global_network_id': network_id,
                                'device_id': conn.get('DeviceId'),
                                'connected_device_id': conn.get('ConnectedDeviceId'),
                                'link_id': conn.get('LinkId'),
                                'connected_link_id': conn.get('ConnectedLinkId'),
                                'description': conn.get('Description'),
                                'state': conn.get('State'),
                                'created_at': str(conn.get('CreatedAt', '')),
                            }

                            conn_tags = {t['Key']: t['Value'] for t in conn.get('Tags', [])}

                            resources.append({
                                'service': 'networkmanager',
                                'type': 'connection',
                                'id': conn_id,
                                'arn': conn_arn,
                                'name': conn.get('Description', conn_id),
                                'region': 'global',
                                'details': conn_details,
                                'tags': conn_tags
                            })
                except Exception:
                    pass
    except Exception:
        pass

    return resources



"""
DynamoDB resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_dynamodb_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect DynamoDB resources: tables, global tables, backups.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    dynamodb = session.client('dynamodb', region_name=region)

    # DynamoDB Tables
    table_names = []
    try:
        paginator = dynamodb.get_paginator('list_tables')
        for page in paginator.paginate():
            table_names.extend(page.get('TableNames', []))
    except Exception:
        pass

    for table_name in table_names:
        try:
            response = dynamodb.describe_table(TableName=table_name)
            table = response.get('Table', {})

            # Get tags
            tags = {}
            try:
                tag_response = dynamodb.list_tags_of_resource(
                    ResourceArn=table['TableArn']
                )
                for tag in tag_response.get('Tags', []):
                    tags[tag.get('Key', '')] = tag.get('Value', '')
            except Exception:
                pass

            # Billing mode
            billing_mode = table.get('BillingModeSummary', {}).get('BillingMode', 'PROVISIONED')

            resources.append({
                'service': 'dynamodb',
                'type': 'table',
                'id': table_name,
                'arn': table['TableArn'],
                'name': table_name,
                'region': region,
                'details': {
                    'status': table.get('TableStatus'),
                    'billing_mode': billing_mode,
                    'item_count': table.get('ItemCount'),
                    'size_bytes': table.get('TableSizeBytes'),
                    'read_capacity': table.get('ProvisionedThroughput', {}).get('ReadCapacityUnits'),
                    'write_capacity': table.get('ProvisionedThroughput', {}).get('WriteCapacityUnits'),
                    'gsi_count': len(table.get('GlobalSecondaryIndexes', [])),
                    'lsi_count': len(table.get('LocalSecondaryIndexes', [])),
                    'stream_enabled': table.get('StreamSpecification', {}).get('StreamEnabled', False),
                    'table_class': table.get('TableClassSummary', {}).get('TableClass'),
                    'deletion_protection': table.get('DeletionProtectionEnabled'),
                    'key_schema': [
                        {k.get('AttributeName'): k.get('KeyType')}
                        for k in table.get('KeySchema', [])
                    ],
                },
                'tags': tags
            })
        except Exception:
            pass

    # DynamoDB Global Tables
    try:
        response = dynamodb.list_global_tables()
        for gt in response.get('GlobalTables', []):
            table_name = gt['GlobalTableName']
            resources.append({
                'service': 'dynamodb',
                'type': 'global-table',
                'id': table_name,
                'arn': f"arn:aws:dynamodb::{account_id}:global-table/{table_name}",
                'name': table_name,
                'region': region,
                'details': {
                    'replication_regions': [r.get('RegionName') for r in gt.get('ReplicationGroup', [])],
                },
                'tags': {}
            })
    except Exception:
        pass

    # DynamoDB Backups
    try:
        paginator = dynamodb.get_paginator('list_backups')
        for page in paginator.paginate():
            for backup in page.get('BackupSummaries', []):
                backup_name = backup['BackupName']
                resources.append({
                    'service': 'dynamodb',
                    'type': 'backup',
                    'id': backup['BackupArn'].split('/')[-1],
                    'arn': backup['BackupArn'],
                    'name': backup_name,
                    'region': region,
                    'details': {
                        'table_name': backup.get('TableName'),
                        'table_arn': backup.get('TableArn'),
                        'backup_status': backup.get('BackupStatus'),
                        'backup_type': backup.get('BackupType'),
                        'backup_size_bytes': backup.get('BackupSizeBytes'),
                        'backup_creation_datetime': str(backup.get('BackupCreationDateTime', '')),
                    },
                    'tags': {}
                })
    except Exception:
        pass

    # DynamoDB Streams
    try:
        streams_client = session.client('dynamodbstreams', region_name=region)
        response = streams_client.list_streams()
        for stream in response.get('Streams', []):
            stream_arn = stream['StreamArn']
            table_name = stream.get('TableName', '')
            stream_label = stream.get('StreamLabel', '')

            # Get stream details
            stream_status = None
            stream_view_type = None
            shard_count = 0
            try:
                desc_response = streams_client.describe_stream(StreamArn=stream_arn)
                stream_desc = desc_response.get('StreamDescription', {})
                stream_status = stream_desc.get('StreamStatus')
                stream_view_type = stream_desc.get('StreamViewType')
                shard_count = len(stream_desc.get('Shards', []))
            except Exception:
                pass

            resources.append({
                'service': 'dynamodb',
                'type': 'stream',
                'id': stream_label,
                'arn': stream_arn,
                'name': f"{table_name}-stream",
                'region': region,
                'details': {
                    'table_name': table_name,
                    'stream_label': stream_label,
                    'stream_status': stream_status,
                    'stream_view_type': stream_view_type,
                    'shard_count': shard_count,
                },
                'tags': {}
            })
    except Exception:
        pass

    return resources



"""
SageMaker resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_sagemaker_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect SageMaker resources: notebook instances, endpoints, models, domains.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    sagemaker = session.client('sagemaker', region_name=region)

    # SageMaker Notebook Instances
    try:
        paginator = sagemaker.get_paginator('list_notebook_instances')
        for page in paginator.paginate():
            for nb in page.get('NotebookInstances', []):
                nb_name = nb['NotebookInstanceName']
                nb_arn = nb['NotebookInstanceArn']

                # Get tags
                tags = {}
                try:
                    tag_response = sagemaker.list_tags(ResourceArn=nb_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'sagemaker',
                    'type': 'notebook-instance',
                    'id': nb_name,
                    'arn': nb_arn,
                    'name': nb_name,
                    'region': region,
                    'details': {
                        'status': nb.get('NotebookInstanceStatus'),
                        'instance_type': nb.get('InstanceType'),
                        'url': nb.get('Url'),
                        'creation_time': str(nb.get('CreationTime', '')),
                        'last_modified_time': str(nb.get('LastModifiedTime', '')),
                        'default_code_repository': nb.get('DefaultCodeRepository'),
                        'additional_code_repositories': nb.get('AdditionalCodeRepositories', []),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # SageMaker Endpoints
    try:
        paginator = sagemaker.get_paginator('list_endpoints')
        for page in paginator.paginate():
            for endpoint in page.get('Endpoints', []):
                endpoint_name = endpoint['EndpointName']
                endpoint_arn = endpoint['EndpointArn']

                # Get tags
                tags = {}
                try:
                    tag_response = sagemaker.list_tags(ResourceArn=endpoint_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'sagemaker',
                    'type': 'endpoint',
                    'id': endpoint_name,
                    'arn': endpoint_arn,
                    'name': endpoint_name,
                    'region': region,
                    'details': {
                        'status': endpoint.get('EndpointStatus'),
                        'creation_time': str(endpoint.get('CreationTime', '')),
                        'last_modified_time': str(endpoint.get('LastModifiedTime', '')),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # SageMaker Models
    try:
        paginator = sagemaker.get_paginator('list_models')
        for page in paginator.paginate():
            for model in page.get('Models', []):
                model_name = model['ModelName']
                model_arn = model['ModelArn']

                # Get tags
                tags = {}
                try:
                    tag_response = sagemaker.list_tags(ResourceArn=model_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'sagemaker',
                    'type': 'model',
                    'id': model_name,
                    'arn': model_arn,
                    'name': model_name,
                    'region': region,
                    'details': {
                        'creation_time': str(model.get('CreationTime', '')),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # SageMaker Domains (Studio)
    try:
        paginator = sagemaker.get_paginator('list_domains')
        for page in paginator.paginate():
            for domain in page.get('Domains', []):
                domain_id = domain['DomainId']
                domain_arn = domain['DomainArn']
                domain_name = domain.get('DomainName', domain_id)

                # Get tags
                tags = {}
                try:
                    tag_response = sagemaker.list_tags(ResourceArn=domain_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'sagemaker',
                    'type': 'domain',
                    'id': domain_id,
                    'arn': domain_arn,
                    'name': domain_name,
                    'region': region,
                    'details': {
                        'status': domain.get('Status'),
                        'creation_time': str(domain.get('CreationTime', '')),
                        'last_modified_time': str(domain.get('LastModifiedTime', '')),
                        'url': domain.get('Url'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # SageMaker Training Jobs (recent active)
    try:
        paginator = sagemaker.get_paginator('list_training_jobs')
        for page in paginator.paginate(StatusEquals='InProgress', MaxResults=100):
            for job in page.get('TrainingJobSummaries', []):
                job_name = job['TrainingJobName']
                job_arn = job['TrainingJobArn']

                # Get tags
                tags = {}
                try:
                    tag_response = sagemaker.list_tags(ResourceArn=job_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'sagemaker',
                    'type': 'training-job',
                    'id': job_name,
                    'arn': job_arn,
                    'name': job_name,
                    'region': region,
                    'details': {
                        'status': job.get('TrainingJobStatus'),
                        'creation_time': str(job.get('CreationTime', '')),
                        'training_end_time': str(job.get('TrainingEndTime', '')),
                        'last_modified_time': str(job.get('LastModifiedTime', '')),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # SageMaker Feature Groups
    try:
        paginator = sagemaker.get_paginator('list_feature_groups')
        for page in paginator.paginate():
            for fg in page.get('FeatureGroupSummaries', []):
                fg_name = fg['FeatureGroupName']
                fg_arn = fg['FeatureGroupArn']

                # Get tags
                tags = {}
                try:
                    tag_response = sagemaker.list_tags(ResourceArn=fg_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'sagemaker',
                    'type': 'feature-group',
                    'id': fg_name,
                    'arn': fg_arn,
                    'name': fg_name,
                    'region': region,
                    'details': {
                        'status': fg.get('FeatureGroupStatus'),
                        'creation_time': str(fg.get('CreationTime', '')),
                        'offline_store_status': fg.get('OfflineStoreStatus', {}).get('Status'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    return resources



"""
AWS Resource Groups resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


def collect_resourcegroups_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Resource Groups.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    rg = session.client('resource-groups', region_name=region)

    # Resource Groups
    try:
        paginator = rg.get_paginator('list_groups')
        for page in paginator.paginate():
            for group in page.get('Groups', []):
                group_arn = group['GroupArn']
                group_name = group['Name']

                details = {
                    'description': group.get('Description'),
                    'criticality': group.get('Criticality'),
                    'owner': group.get('Owner'),
                    'display_name': group.get('DisplayName'),
                }

                # Get group query
                try:
                    query_response = rg.get_group_query(Group=group_name)
                    group_query = query_response.get('GroupQuery', {})
                    resource_query = group_query.get('ResourceQuery', {})
                    details['query_type'] = resource_query.get('Type')
                    details['query'] = resource_query.get('Query')
                except Exception:
                    pass

                # Get group configuration
                try:
                    config_response = rg.get_group_configuration(Group=group_name)
                    config = config_response.get('GroupConfiguration', {})
                    details['configuration_status'] = config.get('Status')
                    config_items = config.get('Configuration', [])
                    details['configuration_types'] = [c.get('Type') for c in config_items]
                except Exception:
                    pass

                # Get tags
                tags = {}
                try:
                    tag_response = rg.get_tags(Arn=group_arn)
                    tags = tag_response.get('Tags', {})
                except Exception:
                    pass

                resources.append({
                    'service': 'resource-groups',
                    'type': 'group',
                    'id': group_name,
                    'arn': group_arn,
                    'name': group.get('DisplayName', group_name),
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    return resources



"""
MemoryDB resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional

# MemoryDB supported regions
# https://docs.aws.amazon.com/general/latest/gr/memorydb-service.html
MEMORYDB_REGIONS = {
    # US
    'us-east-1', 'us-east-2', 'us-west-1', 'us-west-2',
    # Asia Pacific
    'ap-east-1', 'ap-south-1', 'ap-northeast-1', 'ap-northeast-2',
    'ap-southeast-1', 'ap-southeast-2',
    # Canada
    'ca-central-1',
    # Europe
    'eu-central-1', 'eu-west-1', 'eu-west-2', 'eu-west-3',
    'eu-south-1', 'eu-south-2', 'eu-north-1',
    # South America
    'sa-east-1',
}


def collect_memorydb_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect MemoryDB resources: clusters, users, ACLs.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    # Skip unsupported regions to avoid timeouts
    if region and region not in MEMORYDB_REGIONS:
        return []

    resources = []
    memorydb = session.client('memorydb', region_name=region)

    # MemoryDB Clusters
    try:
        paginator = memorydb.get_paginator('describe_clusters')
        for page in paginator.paginate():
            for cluster in page.get('Clusters', []):
                cluster_name = cluster['Name']
                cluster_arn = cluster['ARN']

                # Get tags
                tags = {}
                try:
                    tag_response = memorydb.list_tags(ResourceArn=cluster_arn)
                    for tag in tag_response.get('TagList', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'memorydb',
                    'type': 'cluster',
                    'id': cluster_name,
                    'arn': cluster_arn,
                    'name': cluster_name,
                    'region': region,
                    'details': {
                        'status': cluster.get('Status'),
                        'node_type': cluster.get('NodeType'),
                        'engine_version': cluster.get('EngineVersion'),
                        'engine_patch_version': cluster.get('EnginePatchVersion'),
                        'number_of_shards': cluster.get('NumberOfShards'),
                        'num_replicas_per_shard': cluster.get('Shards', [{}])[0].get('NumberOfNodes', 1) - 1 if cluster.get('Shards') else 0,
                        'availability_mode': cluster.get('AvailabilityMode'),
                        'cluster_endpoint': cluster.get('ClusterEndpoint', {}).get('Address'),
                        'port': cluster.get('ClusterEndpoint', {}).get('Port'),
                        'parameter_group_name': cluster.get('ParameterGroupName'),
                        'subnet_group_name': cluster.get('SubnetGroupName'),
                        'tls_enabled': cluster.get('TLSEnabled'),
                        'kms_key_id': cluster.get('KmsKeyId'),
                        'acl_name': cluster.get('ACLName'),
                        'snapshot_retention_limit': cluster.get('SnapshotRetentionLimit'),
                        'maintenance_window': cluster.get('MaintenanceWindow'),
                        'sns_topic_arn': cluster.get('SnsTopicArn'),
                        'data_tiering': cluster.get('DataTiering'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # MemoryDB ACLs
    try:
        paginator = memorydb.get_paginator('describe_acls')
        for page in paginator.paginate():
            for acl in page.get('ACLs', []):
                acl_name = acl['Name']
                acl_arn = acl['ARN']

                # Skip default ACL
                if acl_name == 'open-access':
                    continue

                # Get tags
                tags = {}
                try:
                    tag_response = memorydb.list_tags(ResourceArn=acl_arn)
                    for tag in tag_response.get('TagList', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'memorydb',
                    'type': 'acl',
                    'id': acl_name,
                    'arn': acl_arn,
                    'name': acl_name,
                    'region': region,
                    'details': {
                        'status': acl.get('Status'),
                        'user_names': acl.get('UserNames', []),
                        'clusters': acl.get('Clusters', []),
                        'minimum_engine_version': acl.get('MinimumEngineVersion'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # MemoryDB Users
    try:
        paginator = memorydb.get_paginator('describe_users')
        for page in paginator.paginate():
            for user in page.get('Users', []):
                user_name = user['Name']
                user_arn = user['ARN']

                # Skip default user
                if user_name == 'default':
                    continue

                # Get tags
                tags = {}
                try:
                    tag_response = memorydb.list_tags(ResourceArn=user_arn)
                    for tag in tag_response.get('TagList', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'memorydb',
                    'type': 'user',
                    'id': user_name,
                    'arn': user_arn,
                    'name': user_name,
                    'region': region,
                    'details': {
                        'status': user.get('Status'),
                        'access_string': user.get('AccessString'),
                        'acl_names': user.get('ACLNames', []),
                        'minimum_engine_version': user.get('MinimumEngineVersion'),
                        'authentication_mode': user.get('Authentication', {}).get('Type'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    return resources



"""
AWS Audit Manager resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


# Audit Manager supported regions (from https://docs.aws.amazon.com/general/latest/gr/audit-manager.html)
AUDITMANAGER_REGIONS = {
    'us-east-1', 'us-east-2', 'us-west-1', 'us-west-2',
    'ap-south-1', 'ap-southeast-1', 'ap-southeast-2', 'ap-northeast-1',
    'ca-central-1',
    'eu-central-1', 'eu-west-1', 'eu-west-2',
}


def collect_auditmanager_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Audit Manager resources: assessments and custom frameworks.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    # Skip unsupported regions
    if region not in AUDITMANAGER_REGIONS:
        return []

    resources = []
    auditmanager = session.client('auditmanager', region_name=region)

    # Assessments
    try:
        paginator = auditmanager.get_paginator('list_assessments')
        for page in paginator.paginate():
            for assessment in page.get('assessmentMetadata', []):
                assessment_id = assessment.get('id', '')
                assessment_name = assessment.get('name', assessment_id)
                assessment_arn = f"arn:aws:auditmanager:{region}:{account_id}:assessment/{assessment_id}"

                details = {
                    'status': assessment.get('status'),
                    'compliance_type': assessment.get('complianceType'),
                    'roles': [r.get('roleName') for r in assessment.get('roles', [])],
                }

                resources.append({
                    'service': 'auditmanager',
                    'type': 'assessment',
                    'id': assessment_id,
                    'arn': assessment_arn,
                    'name': assessment_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Custom Frameworks only
    try:
        paginator = auditmanager.get_paginator('list_assessment_frameworks')
        for page in paginator.paginate(frameworkType='Custom'):
            for framework in page.get('frameworkMetadataList', []):
                framework_id = framework.get('id', '')
                framework_name = framework.get('name', framework_id)
                framework_arn = framework.get('arn', f"arn:aws:auditmanager:{region}:{account_id}:assessmentFramework/{framework_id}")

                details = {
                    'description': framework.get('description'),
                    'compliance_type': framework.get('complianceType'),
                    'controls_count': framework.get('controlsCount'),
                    'control_sets_count': framework.get('controlSetsCount'),
                }

                resources.append({
                    'service': 'auditmanager',
                    'type': 'framework',
                    'id': framework_id,
                    'arn': framework_arn,
                    'name': framework_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
AWS IoT SiteWise resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


# IoT SiteWise supported regions (from https://docs.aws.amazon.com/general/latest/gr/iot-sitewise.html)
IOTSITEWISE_REGIONS = {
    'us-east-1', 'us-east-2', 'us-west-2',
    'ap-south-1', 'ap-northeast-1', 'ap-northeast-2', 'ap-southeast-1', 'ap-southeast-2',
    'ca-central-1',
    'eu-central-1', 'eu-west-1',
}


def collect_iotsitewise_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS IoT SiteWise resources: asset models, assets, gateways, and portals.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    # Skip unsupported regions
    if region not in IOTSITEWISE_REGIONS:
        return []

    resources = []
    sitewise = session.client('iotsitewise', region_name=region)

    # Asset Models
    try:
        paginator = sitewise.get_paginator('list_asset_models')
        for page in paginator.paginate():
            for model in page.get('assetModelSummaries', []):
                model_id = model['id']
                model_arn = model.get('arn', f"arn:aws:iotsitewise:{region}:{account_id}:asset-model/{model_id}")
                model_name = model.get('name', model_id)

                status = model.get('status', {})
                details = {
                    'description': model.get('description'),
                    'asset_model_type': model.get('assetModelType'),
                    'status': status.get('state'),
                    'version': model.get('version'),
                }

                resources.append({
                    'service': 'iotsitewise',
                    'type': 'asset-model',
                    'id': model_id,
                    'arn': model_arn,
                    'name': model_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Assets
    try:
        paginator = sitewise.get_paginator('list_assets')
        for page in paginator.paginate():
            for asset in page.get('assetSummaries', []):
                asset_id = asset['id']
                asset_arn = asset.get('arn', f"arn:aws:iotsitewise:{region}:{account_id}:asset/{asset_id}")
                asset_name = asset.get('name', asset_id)

                status = asset.get('status', {})
                details = {
                    'description': asset.get('description'),
                    'asset_model_id': asset.get('assetModelId'),
                    'status': status.get('state'),
                }

                resources.append({
                    'service': 'iotsitewise',
                    'type': 'asset',
                    'id': asset_id,
                    'arn': asset_arn,
                    'name': asset_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Gateways
    try:
        paginator = sitewise.get_paginator('list_gateways')
        for page in paginator.paginate():
            for gateway in page.get('gatewaySummaries', []):
                gateway_id = gateway['gatewayId']
                gateway_arn = f"arn:aws:iotsitewise:{region}:{account_id}:gateway/{gateway_id}"
                gateway_name = gateway.get('gatewayName', gateway_id)

                platform = gateway.get('gatewayPlatform', {})
                platform_type = None
                if 'greengrass' in platform:
                    platform_type = 'greengrass'
                elif 'greengrassV2' in platform:
                    platform_type = 'greengrassV2'
                elif 'siemensIE' in platform:
                    platform_type = 'siemensIE'

                details = {
                    'gateway_version': gateway.get('gatewayVersion'),
                    'platform_type': platform_type,
                }

                resources.append({
                    'service': 'iotsitewise',
                    'type': 'gateway',
                    'id': gateway_id,
                    'arn': gateway_arn,
                    'name': gateway_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Portals
    try:
        paginator = sitewise.get_paginator('list_portals')
        for page in paginator.paginate():
            for portal in page.get('portalSummaries', []):
                portal_id = portal['id']
                portal_arn = f"arn:aws:iotsitewise:{region}:{account_id}:portal/{portal_id}"
                portal_name = portal.get('name', portal_id)

                status = portal.get('status', {})
                details = {
                    'description': portal.get('description'),
                    'start_url': portal.get('startUrl'),
                    'portal_type': portal.get('portalType'),
                    'status': status.get('state'),
                    'role_arn': portal.get('roleArn'),
                }

                resources.append({
                    'service': 'iotsitewise',
                    'type': 'portal',
                    'id': portal_id,
                    'arn': portal_arn,
                    'name': portal_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
GuardDuty resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_guardduty_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect GuardDuty resources: detectors, IP sets, threat intel sets, filters.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    guardduty = session.client('guardduty', region_name=region)

    # Detectors
    detector_ids = []
    try:
        paginator = guardduty.get_paginator('list_detectors')
        for page in paginator.paginate():
            detector_ids.extend(page.get('DetectorIds', []))
    except Exception:
        pass

    for detector_id in detector_ids:
        try:
            # Get detector details
            response = guardduty.get_detector(DetectorId=detector_id)

            # Get tags
            tags = response.get('Tags', {})

            resources.append({
                'service': 'guardduty',
                'type': 'detector',
                'id': detector_id,
                'arn': f"arn:aws:guardduty:{region}:{account_id}:detector/{detector_id}",
                'name': f"detector-{detector_id[:8]}",
                'region': region,
                'details': {
                    'status': response.get('Status'),
                    'service_role': response.get('ServiceRole'),
                    'created_at': str(response.get('CreatedAt', '')),
                    'updated_at': str(response.get('UpdatedAt', '')),
                    'finding_publishing_frequency': response.get('FindingPublishingFrequency'),
                    'data_sources': response.get('DataSources'),
                    'features': response.get('Features'),
                },
                'tags': tags
            })

            # IP Sets for this detector
            try:
                ipset_paginator = guardduty.get_paginator('list_ip_sets')
                for ipset_page in ipset_paginator.paginate(DetectorId=detector_id):
                    for ipset_id in ipset_page.get('IpSetIds', []):
                        try:
                            ipset_response = guardduty.get_ip_set(
                                DetectorId=detector_id,
                                IpSetId=ipset_id
                            )

                            ipset_tags = ipset_response.get('Tags', {})

                            resources.append({
                                'service': 'guardduty',
                                'type': 'ip-set',
                                'id': ipset_id,
                                'arn': f"arn:aws:guardduty:{region}:{account_id}:detector/{detector_id}/ipset/{ipset_id}",
                                'name': ipset_response.get('Name', ipset_id),
                                'region': region,
                                'details': {
                                    'detector_id': detector_id,
                                    'format': ipset_response.get('Format'),
                                    'location': ipset_response.get('Location'),
                                    'status': ipset_response.get('Status'),
                                },
                                'tags': ipset_tags
                            })
                        except Exception:
                            pass
            except Exception:
                pass

            # Threat Intel Sets for this detector
            try:
                ti_paginator = guardduty.get_paginator('list_threat_intel_sets')
                for ti_page in ti_paginator.paginate(DetectorId=detector_id):
                    for ti_id in ti_page.get('ThreatIntelSetIds', []):
                        try:
                            ti_response = guardduty.get_threat_intel_set(
                                DetectorId=detector_id,
                                ThreatIntelSetId=ti_id
                            )

                            ti_tags = ti_response.get('Tags', {})

                            resources.append({
                                'service': 'guardduty',
                                'type': 'threat-intel-set',
                                'id': ti_id,
                                'arn': f"arn:aws:guardduty:{region}:{account_id}:detector/{detector_id}/threatintelset/{ti_id}",
                                'name': ti_response.get('Name', ti_id),
                                'region': region,
                                'details': {
                                    'detector_id': detector_id,
                                    'format': ti_response.get('Format'),
                                    'location': ti_response.get('Location'),
                                    'status': ti_response.get('Status'),
                                },
                                'tags': ti_tags
                            })
                        except Exception:
                            pass
            except Exception:
                pass

            # Filters for this detector
            try:
                filter_paginator = guardduty.get_paginator('list_filters')
                for filter_page in filter_paginator.paginate(DetectorId=detector_id):
                    for filter_name in filter_page.get('FilterNames', []):
                        try:
                            filter_response = guardduty.get_filter(
                                DetectorId=detector_id,
                                FilterName=filter_name
                            )

                            filter_tags = filter_response.get('Tags', {})

                            resources.append({
                                'service': 'guardduty',
                                'type': 'filter',
                                'id': filter_name,
                                'arn': f"arn:aws:guardduty:{region}:{account_id}:detector/{detector_id}/filter/{filter_name}",
                                'name': filter_name,
                                'region': region,
                                'details': {
                                    'detector_id': detector_id,
                                    'description': filter_response.get('Description'),
                                    'rank': filter_response.get('Rank'),
                                    'action': filter_response.get('Action'),
                                },
                                'tags': filter_tags
                            })
                        except Exception:
                            pass
            except Exception:
                pass

        except Exception:
            pass

    return resources



"""
Kinesis Data Firehose resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_firehose_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect Kinesis Data Firehose delivery streams.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    firehose = session.client('firehose', region_name=region)

    try:
        # List all delivery streams
        stream_names = []
        response = firehose.list_delivery_streams()
        stream_names.extend(response.get('DeliveryStreamNames', []))

        while response.get('HasMoreDeliveryStreams'):
            response = firehose.list_delivery_streams(
                ExclusiveStartDeliveryStreamName=stream_names[-1]
            )
            stream_names.extend(response.get('DeliveryStreamNames', []))

        # Describe each stream
        for stream_name in stream_names:
            try:
                stream_response = firehose.describe_delivery_stream(
                    DeliveryStreamName=stream_name
                )
                stream_desc = stream_response.get('DeliveryStreamDescription', {})

                stream_arn = stream_desc.get('DeliveryStreamARN')

                # Get tags
                tags = {}
                try:
                    tag_response = firehose.list_tags_for_delivery_stream(
                        DeliveryStreamName=stream_name
                    )
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                # Determine destination type
                destinations = stream_desc.get('Destinations', [])
                destination_types = []
                for dest in destinations:
                    if dest.get('S3DestinationDescription'):
                        destination_types.append('S3')
                    if dest.get('RedshiftDestinationDescription'):
                        destination_types.append('Redshift')
                    if dest.get('ElasticsearchDestinationDescription'):
                        destination_types.append('Elasticsearch')
                    if dest.get('HttpEndpointDestinationDescription'):
                        destination_types.append('HTTP')
                    if dest.get('SplunkDestinationDescription'):
                        destination_types.append('Splunk')
                    if dest.get('AmazonOpenSearchServerlessDestinationDescription'):
                        destination_types.append('OpenSearchServerless')
                    if dest.get('AmazonopensearchserviceDestinationDescription'):
                        destination_types.append('OpenSearch')

                resources.append({
                    'service': 'firehose',
                    'type': 'delivery-stream',
                    'id': stream_name,
                    'arn': stream_arn,
                    'name': stream_name,
                    'region': region,
                    'details': {
                        'status': stream_desc.get('DeliveryStreamStatus'),
                        'delivery_stream_type': stream_desc.get('DeliveryStreamType'),
                        'version_id': stream_desc.get('VersionId'),
                        'destination_types': destination_types,
                        'source_kinesis_stream_arn': stream_desc.get('Source', {}).get('KinesisStreamSourceDescription', {}).get('KinesisStreamARN'),
                        'create_timestamp': str(stream_desc.get('CreateTimestamp', '')),
                        'last_update_timestamp': str(stream_desc.get('LastUpdateTimestamp', '')),
                    },
                    'tags': tags
                })
            except Exception:
                pass
    except Exception:
        pass

    return resources



"""
Directory Service resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_ds_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect Directory Service resources: directories.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    ds = session.client('ds', region_name=region)

    # Directories
    try:
        paginator = ds.get_paginator('describe_directories')
        for page in paginator.paginate():
            for directory in page.get('DirectoryDescriptions', []):
                dir_id = directory['DirectoryId']

                # Get tags
                tags = {}
                try:
                    tag_response = ds.list_tags_for_resource(ResourceId=dir_id)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'ds',
                    'type': 'directory',
                    'id': dir_id,
                    'arn': f"arn:aws:ds:{region}:{account_id}:directory/{dir_id}",
                    'name': directory.get('Name', dir_id),
                    'region': region,
                    'details': {
                        'short_name': directory.get('ShortName'),
                        'type': directory.get('Type'),
                        'edition': directory.get('Edition'),
                        'size': directory.get('Size'),
                        'alias': directory.get('Alias'),
                        'access_url': directory.get('AccessUrl'),
                        'stage': directory.get('Stage'),
                        'stage_reason': directory.get('StageReason'),
                        'launch_time': str(directory.get('LaunchTime', '')),
                        'stage_last_updated_date_time': str(directory.get('StageLastUpdatedDateTime', '')),
                        'sso_enabled': directory.get('SsoEnabled'),
                        'vpc_settings': {
                            'vpc_id': directory.get('VpcSettings', {}).get('VpcId'),
                            'subnet_ids': directory.get('VpcSettings', {}).get('SubnetIds', []),
                            'availability_zones': directory.get('VpcSettings', {}).get('AvailabilityZones', []),
                        },
                        'dns_ip_addrs': directory.get('DnsIpAddrs', []),
                        'connect_settings': bool(directory.get('ConnectSettings')),
                        'radius_settings': bool(directory.get('RadiusSettings')),
                        'desired_number_of_domain_controllers': directory.get('DesiredNumberOfDomainControllers'),
                        'owner_directory_description': bool(directory.get('OwnerDirectoryDescription')),
                        'regions_info': directory.get('RegionsInfo'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    return resources



"""
CloudFront resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_cloudfront_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect CloudFront resources: distributions, functions, origin access identities.

    Args:
        session: boto3.Session to use
        region: Not used for CloudFront (global service)
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    cloudfront = session.client('cloudfront')

    # CloudFront Distributions
    try:
        paginator = cloudfront.get_paginator('list_distributions')
        for page in paginator.paginate():
            dist_list = page.get('DistributionList', {})
            for dist in dist_list.get('Items', []):
                dist_id = dist['Id']
                dist_arn = dist['ARN']

                # Get tags
                tags = {}
                try:
                    tag_response = cloudfront.list_tags_for_resource(Resource=dist_arn)
                    for tag in tag_response.get('Tags', {}).get('Items', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                origins = dist.get('Origins', {}).get('Items', [])
                aliases = dist.get('Aliases', {}).get('Items', [])

                resources.append({
                    'service': 'cloudfront',
                    'type': 'distribution',
                    'id': dist_id,
                    'arn': dist_arn,
                    'name': tags.get('Name') or aliases[0] if aliases else dist_id,
                    'region': 'global',
                    'details': {
                        'domain_name': dist.get('DomainName'),
                        'status': dist.get('Status'),
                        'enabled': dist.get('Enabled'),
                        'aliases': aliases,
                        'origins': [o.get('DomainName') for o in origins],
                        'price_class': dist.get('PriceClass'),
                        'http_version': dist.get('HttpVersion'),
                        'is_ipv6_enabled': dist.get('IsIPV6Enabled'),
                        'default_root_object': dist.get('DefaultRootObject'),
                        'comment': dist.get('Comment'),
                        'web_acl_id': dist.get('WebACLId'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # CloudFront Functions
    try:
        paginator = cloudfront.get_paginator('list_functions')
        for page in paginator.paginate():
            func_list = page.get('FunctionList', {})
            for func in func_list.get('Items', []):
                func_name = func['Name']
                func_arn = func['FunctionMetadata']['FunctionARN']

                resources.append({
                    'service': 'cloudfront',
                    'type': 'function',
                    'id': func_name,
                    'arn': func_arn,
                    'name': func_name,
                    'region': 'global',
                    'details': {
                        'status': func.get('Status'),
                        'stage': func.get('FunctionMetadata', {}).get('Stage'),
                        'created_time': str(func.get('FunctionMetadata', {}).get('CreatedTime', '')),
                        'last_modified_time': str(func.get('FunctionMetadata', {}).get('LastModifiedTime', '')),
                        'comment': func.get('FunctionConfig', {}).get('Comment'),
                        'runtime': func.get('FunctionConfig', {}).get('Runtime'),
                    },
                    'tags': {}
                })
    except Exception:
        pass

    # Origin Access Identities
    try:
        paginator = cloudfront.get_paginator('list_cloud_front_origin_access_identities')
        for page in paginator.paginate():
            oai_list = page.get('CloudFrontOriginAccessIdentityList', {})
            for oai in oai_list.get('Items', []):
                oai_id = oai['Id']

                resources.append({
                    'service': 'cloudfront',
                    'type': 'origin-access-identity',
                    'id': oai_id,
                    'arn': f"arn:aws:cloudfront::{account_id}:origin-access-identity/{oai_id}",
                    'name': oai.get('Comment') or oai_id,
                    'region': 'global',
                    'details': {
                        's3_canonical_user_id': oai.get('S3CanonicalUserId'),
                        'comment': oai.get('Comment'),
                    },
                    'tags': {}
                })
    except Exception:
        pass

    # Origin Access Controls
    try:
        response = cloudfront.list_origin_access_controls()
        oac_list = response.get('OriginAccessControlList', {})
        for oac in oac_list.get('Items', []):
            oac_id = oac['Id']

            resources.append({
                'service': 'cloudfront',
                'type': 'origin-access-control',
                'id': oac_id,
                'arn': f"arn:aws:cloudfront::{account_id}:origin-access-control/{oac_id}",
                'name': oac.get('Name') or oac_id,
                'region': 'global',
                'details': {
                    'description': oac.get('Description'),
                    'origin_access_control_origin_type': oac.get('OriginAccessControlOriginType'),
                    'signing_behavior': oac.get('SigningBehavior'),
                    'signing_protocol': oac.get('SigningProtocol'),
                },
                'tags': {}
            })
    except Exception:
        pass

    # Cache Policies (custom only)
    try:
        response = cloudfront.list_cache_policies(Type='custom')
        policy_list = response.get('CachePolicyList', {})
        for policy in policy_list.get('Items', []):
            cp = policy.get('CachePolicy', {})
            cp_id = cp.get('Id')
            if not cp_id:
                continue

            resources.append({
                'service': 'cloudfront',
                'type': 'cache-policy',
                'id': cp_id,
                'arn': f"arn:aws:cloudfront::{account_id}:cache-policy/{cp_id}",
                'name': cp.get('CachePolicyConfig', {}).get('Name') or cp_id,
                'region': 'global',
                'details': {
                    'comment': cp.get('CachePolicyConfig', {}).get('Comment'),
                    'default_ttl': cp.get('CachePolicyConfig', {}).get('DefaultTTL'),
                    'max_ttl': cp.get('CachePolicyConfig', {}).get('MaxTTL'),
                    'min_ttl': cp.get('CachePolicyConfig', {}).get('MinTTL'),
                },
                'tags': {}
            })
    except Exception:
        pass

    return resources



"""
AWS Transcribe resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


def collect_transcribe_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Transcribe resources: vocabularies, vocabulary filters, and language models.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    transcribe = session.client('transcribe', region_name=region)

    # Vocabularies
    try:
        paginator = transcribe.get_paginator('list_vocabularies')
        for page in paginator.paginate():
            for vocab in page.get('Vocabularies', []):
                vocab_name = vocab['VocabularyName']

                details = {
                    'language_code': vocab.get('LanguageCode'),
                    'vocabulary_state': vocab.get('VocabularyState'),
                    'last_modified_time': str(vocab.get('LastModifiedTime', '')),
                }

                resources.append({
                    'service': 'transcribe',
                    'type': 'vocabulary',
                    'id': vocab_name,
                    'arn': f"arn:aws:transcribe:{region}:{account_id}:vocabulary/{vocab_name}",
                    'name': vocab_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Vocabulary Filters
    try:
        paginator = transcribe.get_paginator('list_vocabulary_filters')
        for page in paginator.paginate():
            for filter_item in page.get('VocabularyFilters', []):
                filter_name = filter_item['VocabularyFilterName']

                details = {
                    'language_code': filter_item.get('LanguageCode'),
                    'last_modified_time': str(filter_item.get('LastModifiedTime', '')),
                }

                resources.append({
                    'service': 'transcribe',
                    'type': 'vocabulary-filter',
                    'id': filter_name,
                    'arn': f"arn:aws:transcribe:{region}:{account_id}:vocabulary-filter/{filter_name}",
                    'name': filter_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Language Models
    try:
        paginator = transcribe.get_paginator('list_language_models')
        for page in paginator.paginate():
            for model in page.get('Models', []):
                model_name = model['ModelName']

                details = {
                    'language_code': model.get('LanguageCode'),
                    'base_model_name': model.get('BaseModelName'),
                    'model_status': model.get('ModelStatus'),
                    'create_time': str(model.get('CreateTime', '')),
                    'last_modified_time': str(model.get('LastModifiedTime', '')),
                    'upgrade_availability': model.get('UpgradeAvailability'),
                }

                input_data = model.get('InputDataConfig', {})
                if input_data:
                    details['s3_uri'] = input_data.get('S3Uri')
                    details['data_access_role_arn'] = input_data.get('DataAccessRoleArn')

                resources.append({
                    'service': 'transcribe',
                    'type': 'language-model',
                    'id': model_name,
                    'arn': f"arn:aws:transcribe:{region}:{account_id}:language-model/{model_name}",
                    'name': model_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Call Analytics Categories
    try:
        paginator = transcribe.get_paginator('list_call_analytics_categories')
        for page in paginator.paginate():
            for category in page.get('Categories', []):
                category_name = category['CategoryName']

                details = {
                    'create_time': str(category.get('CreateTime', '')),
                    'last_update_time': str(category.get('LastUpdateTime', '')),
                    'input_type': category.get('InputType'),
                }

                resources.append({
                    'service': 'transcribe',
                    'type': 'call-analytics-category',
                    'id': category_name,
                    'arn': f"arn:aws:transcribe:{region}:{account_id}:analytics-category/{category_name}",
                    'name': category_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
AWS Interactive Video Service (IVS) resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


# IVS supported regions (from https://docs.aws.amazon.com/general/latest/gr/ivs.html)
IVS_REGIONS = {
    'us-east-1', 'us-west-2',
    'ap-south-1', 'ap-northeast-1', 'ap-northeast-2',
    'eu-central-1', 'eu-west-1',
}


def collect_ivs_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS IVS resources: channels, playback key pairs, and recording configurations.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    # Skip unsupported regions
    if region not in IVS_REGIONS:
        return []

    resources = []
    ivs = session.client('ivs', region_name=region)

    # Channels
    try:
        paginator = ivs.get_paginator('list_channels')
        for page in paginator.paginate():
            for channel in page.get('channels', []):
                channel_arn = channel['arn']
                channel_name = channel.get('name', channel_arn.split('/')[-1])

                details = {
                    'latency_mode': channel.get('latencyMode'),
                    'authorized': channel.get('authorized'),
                    'recording_configuration_arn': channel.get('recordingConfigurationArn'),
                    'type': channel.get('type'),
                    'preset': channel.get('preset'),
                    'playback_restriction_policy_arn': channel.get('playbackRestrictionPolicyArn'),
                }

                tags = channel.get('tags', {})

                resources.append({
                    'service': 'ivs',
                    'type': 'channel',
                    'id': channel_name,
                    'arn': channel_arn,
                    'name': channel_name,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Playback Key Pairs
    try:
        paginator = ivs.get_paginator('list_playback_key_pairs')
        for page in paginator.paginate():
            for pkp in page.get('keyPairs', []):
                pkp_arn = pkp['arn']
                pkp_name = pkp.get('name', pkp_arn.split('/')[-1])

                details = {
                    'fingerprint': pkp.get('fingerprint'),
                }

                tags = pkp.get('tags', {})

                resources.append({
                    'service': 'ivs',
                    'type': 'playback-key-pair',
                    'id': pkp_name,
                    'arn': pkp_arn,
                    'name': pkp_name,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Recording Configurations
    try:
        paginator = ivs.get_paginator('list_recording_configurations')
        for page in paginator.paginate():
            for rc in page.get('recordingConfigurations', []):
                rc_arn = rc['arn']
                rc_name = rc.get('name', rc_arn.split('/')[-1])

                details = {
                    'state': rc.get('state'),
                    'destination_configuration': rc.get('destinationConfiguration', {}),
                }

                tags = rc.get('tags', {})

                resources.append({
                    'service': 'ivs',
                    'type': 'recording-configuration',
                    'id': rc_name,
                    'arn': rc_arn,
                    'name': rc_name,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    return resources



"""
SNS resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_sns_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect SNS resources: topics, subscriptions.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    sns = session.client('sns', region_name=region)

    # SNS Topics
    topics = []
    try:
        paginator = sns.get_paginator('list_topics')
        for page in paginator.paginate():
            for topic in page.get('Topics', []):
                topics.append(topic['TopicArn'])
    except Exception:
        pass

    for topic_arn in topics:
        try:
            # Get topic attributes
            attr_response = sns.get_topic_attributes(TopicArn=topic_arn)
            attributes = attr_response.get('Attributes', {})

            topic_name = topic_arn.split(':')[-1]

            # Get tags
            tags = {}
            try:
                tag_response = sns.list_tags_for_resource(ResourceArn=topic_arn)
                for tag in tag_response.get('Tags', []):
                    tags[tag.get('Key', '')] = tag.get('Value', '')
            except Exception:
                pass

            resources.append({
                'service': 'sns',
                'type': 'topic',
                'id': topic_name,
                'arn': topic_arn,
                'name': attributes.get('DisplayName') or topic_name,
                'region': region,
                'details': {
                    'display_name': attributes.get('DisplayName'),
                    'subscriptions_confirmed': int(attributes.get('SubscriptionsConfirmed', 0)),
                    'subscriptions_pending': int(attributes.get('SubscriptionsPending', 0)),
                    'subscriptions_deleted': int(attributes.get('SubscriptionsDeleted', 0)),
                    'delivery_policy': attributes.get('DeliveryPolicy'),
                    'effective_delivery_policy': attributes.get('EffectiveDeliveryPolicy'),
                    'kms_master_key_id': attributes.get('KmsMasterKeyId'),
                    'fifo_topic': attributes.get('FifoTopic') == 'true',
                    'content_based_deduplication': attributes.get('ContentBasedDeduplication') == 'true',
                },
                'tags': tags
            })
        except Exception:
            pass

    # SNS Subscriptions
    try:
        paginator = sns.get_paginator('list_subscriptions')
        for page in paginator.paginate():
            for sub in page.get('Subscriptions', []):
                sub_arn = sub.get('SubscriptionArn', '')

                # Skip pending confirmations
                if sub_arn == 'PendingConfirmation':
                    continue

                topic_arn = sub.get('TopicArn', '')
                topic_name = topic_arn.split(':')[-1] if topic_arn else 'unknown'

                resources.append({
                    'service': 'sns',
                    'type': 'subscription',
                    'id': sub_arn.split(':')[-1] if ':' in sub_arn else sub_arn,
                    'arn': sub_arn,
                    'name': f"{topic_name}-{sub.get('Protocol', '')}",
                    'region': region,
                    'details': {
                        'topic_arn': topic_arn,
                        'protocol': sub.get('Protocol'),
                        'endpoint': sub.get('Endpoint'),
                        'owner': sub.get('Owner'),
                    },
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
AWS Elemental MediaConnect resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_mediaconnect_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Elemental MediaConnect resources: flows, bridges, gateways.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    mediaconnect = session.client('mediaconnect', region_name=region)

    # Flows
    try:
        paginator = mediaconnect.get_paginator('list_flows')
        for page in paginator.paginate():
            for flow in page.get('Flows', []):
                flow_arn = flow.get('FlowArn', '')
                flow_name = flow.get('Name', flow_arn.split('/')[-1])

                details = {
                    'status': flow.get('Status'),
                    'availability_zone': flow.get('AvailabilityZone'),
                    'description': flow.get('Description'),
                    'source_type': flow.get('SourceType'),
                }

                resources.append({
                    'service': 'mediaconnect',
                    'type': 'flow',
                    'id': flow_name,
                    'arn': flow_arn,
                    'name': flow_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Gateways
    try:
        paginator = mediaconnect.get_paginator('list_gateways')
        for page in paginator.paginate():
            for gateway in page.get('Gateways', []):
                gateway_arn = gateway.get('GatewayArn', '')
                gateway_name = gateway.get('Name', gateway_arn.split('/')[-1])

                details = {
                    'gateway_state': gateway.get('GatewayState'),
                }

                resources.append({
                    'service': 'mediaconnect',
                    'type': 'gateway',
                    'id': gateway_name,
                    'arn': gateway_arn,
                    'name': gateway_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Bridges
    try:
        paginator = mediaconnect.get_paginator('list_bridges')
        for page in paginator.paginate():
            for bridge in page.get('Bridges', []):
                bridge_arn = bridge.get('BridgeArn', '')
                bridge_name = bridge.get('Name', bridge_arn.split('/')[-1])

                details = {
                    'bridge_state': bridge.get('BridgeState'),
                    'bridge_type': bridge.get('BridgeType'),
                    'placement_arn': bridge.get('PlacementArn'),
                }

                resources.append({
                    'service': 'mediaconnect',
                    'type': 'bridge',
                    'id': bridge_name,
                    'arn': bridge_arn,
                    'name': bridge_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
AWS DynamoDB Accelerator (DAX) resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


# DAX supported regions (from https://docs.aws.amazon.com/general/latest/gr/ddb.html)
DAX_REGIONS = {
    'us-east-1', 'us-east-2', 'us-west-1', 'us-west-2',
    'ap-south-1', 'ap-southeast-1', 'ap-southeast-2', 'ap-northeast-1',
    'eu-central-1', 'eu-west-1', 'eu-west-2', 'eu-west-3', 'eu-south-2', 'eu-north-1',
    'sa-east-1',
}


def collect_dax_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS DAX resources: clusters, parameter groups, and subnet groups.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    # Skip unsupported regions
    if region not in DAX_REGIONS:
        return []

    resources = []
    dax = session.client('dax', region_name=region)

    # Clusters
    try:
        paginator = dax.get_paginator('describe_clusters')
        for page in paginator.paginate():
            for cluster in page.get('Clusters', []):
                cluster_name = cluster['ClusterName']
                cluster_arn = cluster.get('ClusterArn', '')

                details = {
                    'description': cluster.get('Description'),
                    'status': cluster.get('Status'),
                    'node_type': cluster.get('NodeType'),
                    'total_nodes': cluster.get('TotalNodes'),
                    'active_nodes': cluster.get('ActiveNodes'),
                    'cluster_discovery_endpoint': cluster.get('ClusterDiscoveryEndpoint', {}).get('Address'),
                    'cluster_discovery_port': cluster.get('ClusterDiscoveryEndpoint', {}).get('Port'),
                    'preferred_maintenance_window': cluster.get('PreferredMaintenanceWindow'),
                    'subnet_group': cluster.get('SubnetGroup'),
                    'security_groups': [sg.get('SecurityGroupIdentifier') for sg in cluster.get('SecurityGroups', [])],
                    'iam_role_arn': cluster.get('IamRoleArn'),
                    'parameter_group': cluster.get('ParameterGroup', {}).get('ParameterGroupName'),
                    'sse_enabled': cluster.get('SSEDescription', {}).get('Status') == 'ENABLED',
                    'cluster_endpoint_encryption_type': cluster.get('ClusterEndpointEncryptionType'),
                }

                # Node information
                nodes = cluster.get('Nodes', [])
                details['node_ids'] = [n.get('NodeId') for n in nodes]
                details['node_count'] = len(nodes)

                # Get tags
                tags = {}
                try:
                    tag_response = dax.list_tags(ResourceName=cluster_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'dax',
                    'type': 'cluster',
                    'id': cluster_name,
                    'arn': cluster_arn,
                    'name': cluster_name,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Parameter Groups (only custom, skip default)
    try:
        paginator = dax.get_paginator('describe_parameter_groups')
        for page in paginator.paginate():
            for pg in page.get('ParameterGroups', []):
                pg_name = pg['ParameterGroupName']

                # Skip default parameter group
                if pg_name == 'default.dax1.0':
                    continue

                details = {
                    'description': pg.get('Description'),
                }

                resources.append({
                    'service': 'dax',
                    'type': 'parameter-group',
                    'id': pg_name,
                    'arn': f"arn:aws:dax:{region}:{account_id}:parameter-group/{pg_name}",
                    'name': pg_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Subnet Groups
    try:
        paginator = dax.get_paginator('describe_subnet_groups')
        for page in paginator.paginate():
            for sg in page.get('SubnetGroups', []):
                sg_name = sg['SubnetGroupName']

                details = {
                    'description': sg.get('Description'),
                    'vpc_id': sg.get('VpcId'),
                    'subnets': [s.get('SubnetIdentifier') for s in sg.get('Subnets', [])],
                    'subnet_count': len(sg.get('Subnets', [])),
                }

                resources.append({
                    'service': 'dax',
                    'type': 'subnet-group',
                    'id': sg_name,
                    'arn': f"arn:aws:dax:{region}:{account_id}:subnet-group/{sg_name}",
                    'name': sg_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
Command-line interface for AWS Inventory Tool.
"""

import sys
import time
import click
from typing import Optional, List

from aws_inventory.auth import create_session, validate_credentials, get_account_alias
from aws_inventory.collector import collect_all, get_available_services, validate_services
from aws_inventory.formatter import format_output, export_file


def print_progress(service: str, status: str) -> None:
    """Print progress update for a service."""
    click.echo(f"  {service.upper():20} {status}")


@click.command()
@click.option('--profile', '-p', default=None, help='AWS profile name to use')
@click.option('--region', '-r', multiple=True, help='AWS region(s) to scan (can be specified multiple times)')
@click.option('--services', '-s', multiple=True, help='Service(s) to scan (can be specified multiple times)')
@click.option('--format', '-f', 'output_format', type=click.Choice(['json', 'csv', 'html']), default='html', help='Output format')
@click.option('--output', '-o', 'output_file', default=None, help='Output file path (auto-generated if not specified)')
@click.option('--workers', '-w', default=40, type=int, help='Maximum parallel workers (default: 40)')
@click.option('--list-services', is_flag=True, help='List available service collectors')
@click.option('--tag', '-t', multiple=True, help='Filter by tag (Key=Value format, can be specified multiple times)')
@click.option('--quiet', '-q', is_flag=True, help='Suppress progress output')
@click.option('--timings', is_flag=True, help='Show timing summary per service')
@click.option('--include-global', is_flag=True, help='Include global services even when filtering by non-global regions')
@click.option('--exclude-defaults', is_flag=True, help='Exclude default AWS resources (default VPCs, security groups, etc.)')
def main(
    profile: Optional[str],
    region: tuple,
    services: tuple,
    output_format: str,
    output_file: Optional[str],
    workers: int,
    list_services: bool,
    tag: tuple,
    quiet: bool,
    timings: bool,
    include_global: bool,
    exclude_defaults: bool
) -> None:
    """
    awsmap - Map and inventory AWS resources.

    Examples:

        # Scan all services in all regions (HTML output)
        awsmap

        # Scan specific services
        awsmap -s ec2 -s s3 -s rds

        # Scan specific regions
        awsmap -r us-east-1 -r eu-west-1

        # Use specific AWS profile
        awsmap -p production

        # Filter by tags
        awsmap -t Owner=Tarek -t Environment=Production

        # Output as JSON
        awsmap -f json -o inventory.json

        # List available collectors
        awsmap --list-services
    """
    # List services mode
    if list_services:
        services = get_available_services()
        click.echo(f"\nAvailable service collectors ({len(services)}):\n")
        for i, svc in enumerate(services, 1):
            click.echo(f"  {i:3}. {svc}")
        click.echo()
        return

    # Parse and validate services early (no AWS credentials needed)
    services_list: Optional[List[str]] = None
    if services:
        services_list = []
        for s in services:
            services_list.extend([x.strip() for x in s.split(',')])
        try:
            validate_services(services_list)
        except ValueError as e:
            click.echo(f"Error: {e}", err=True)
            sys.exit(1)

    # Create session
    try:
        session = create_session(profile_name=profile)
    except Exception as e:
        click.echo(f"Error creating session: {e}", err=True)
        sys.exit(1)

    # Validate credentials
    if not quiet:
        click.echo("\nValidating AWS credentials...")

    try:
        identity = validate_credentials(session)
        account_id = identity['account_id']
        account_alias = get_account_alias(session)

        if not quiet:
            click.echo(f"  Account ID: {account_id}")
            if account_alias:
                click.echo(f"  Account Alias: {account_alias}")
            click.echo(f"  User ARN: {identity['arn']}")
    except ValueError as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)

    # Parse regions (support both -r us-east-1 -r eu-west-1 and -r us-east-1,eu-west-1)
    regions_list: Optional[List[str]] = None
    if region:
        regions_list = []
        for r in region:
            regions_list.extend([x.strip() for x in r.split(',')])

    # Run collection
    if not quiet:
        click.echo("\nCollecting resources...")
        click.echo("-" * 40)

    start_time = time.time()

    progress_callback = None if quiet else print_progress

    try:
        result = collect_all(
            session=session,
            services=services_list,
            regions=regions_list,
            max_workers=workers,
            progress_callback=progress_callback,
            show_timings=timings,
            include_global=include_global
        )
    except Exception as e:
        click.echo(f"Error during collection: {e}", err=True)
        sys.exit(1)

    elapsed = time.time() - start_time

    # Apply tag filters (same key = OR, different keys = AND)
    if tag:
        tag_filters = {}
        for t in tag:
            if '=' in t:
                key, value = t.split('=', 1)
                if key not in tag_filters:
                    tag_filters[key] = []
                tag_filters[key].append(value)

        if tag_filters:
            filtered_resources = []
            for resource in result['resources']:
                resource_tags = resource.get('tags', {})
                # All keys must match (AND), but values are OR within same key
                match = all(
                    resource_tags.get(k) in values
                    for k, values in tag_filters.items()
                )
                if match:
                    filtered_resources.append(resource)

            result['resources'] = filtered_resources
            result['metadata']['resource_count'] = len(filtered_resources)
            result['metadata']['tag_filter'] = tag_filters

    # Exclude default AWS resources
    if exclude_defaults:
        result['resources'] = [r for r in result['resources'] if not r.get('is_default')]
        result['metadata']['resource_count'] = len(result['resources'])
        result['metadata']['exclude_defaults'] = True

    # Summary
    if not quiet:
        click.echo("-" * 40)
        click.echo(f"\nCollection complete!")
        click.echo(f"  Resources found: {result['metadata']['resource_count']:,}")
        click.echo(f"  Services scanned: {result['metadata']['services_scanned']}")
        click.echo(f"  Regions scanned: {result['metadata']['regions_scanned']}")
        click.echo(f"  Duration: {elapsed:.1f}s")

    # Format output
    try:
        output_content = format_output(result, output_format)
    except Exception as e:
        click.echo(f"Error formatting output: {e}", err=True)
        sys.exit(1)

    # Determine output file path
    if not output_file:
        timestamp = time.strftime('%Y%m%d_%H%M%S')
        ext = output_format
        output_file = f"{account_id}_inventory_{timestamp}.{ext}"

    # Write output
    try:
        export_file(output_content, output_file)
        if not quiet:
            click.echo(f"\nOutput saved to: {output_file}")
    except Exception as e:
        click.echo(f"Error writing output: {e}", err=True)
        sys.exit(1)


if __name__ == '__main__':
    main()



"""
Bedrock resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_bedrock_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect Bedrock resources: custom models, model customization jobs, provisioned throughput.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    bedrock = session.client('bedrock', region_name=region)

    # Custom Models
    try:
        paginator = bedrock.get_paginator('list_custom_models')
        for page in paginator.paginate():
            for model in page.get('modelSummaries', []):
                model_arn = model['modelArn']
                model_name = model['modelName']

                try:
                    # Get model details
                    model_response = bedrock.get_custom_model(modelIdentifier=model_arn)

                    resources.append({
                        'service': 'bedrock',
                        'type': 'custom-model',
                        'id': model_name,
                        'arn': model_arn,
                        'name': model_name,
                        'region': region,
                        'details': {
                            'base_model_arn': model_response.get('baseModelArn'),
                            'customization_type': model_response.get('customizationType'),
                            'creation_time': str(model_response.get('creationTime', '')),
                            'job_arn': model_response.get('jobArn'),
                            'training_data_config': model_response.get('trainingDataConfig'),
                            'output_data_config': model_response.get('outputDataConfig'),
                        },
                        'tags': {}
                    })
                except Exception:
                    pass
    except Exception:
        pass

    # Model Customization Jobs (active)
    try:
        paginator = bedrock.get_paginator('list_model_customization_jobs')
        for page in paginator.paginate():
            for job in page.get('modelCustomizationJobSummaries', []):
                job_arn = job['jobArn']
                job_name = job['jobName']

                # Only include active jobs
                status = job.get('status')
                if status in ['Completed', 'Failed', 'Stopped']:
                    continue

                resources.append({
                    'service': 'bedrock',
                    'type': 'customization-job',
                    'id': job_name,
                    'arn': job_arn,
                    'name': job_name,
                    'region': region,
                    'details': {
                        'status': status,
                        'base_model_arn': job.get('baseModelArn'),
                        'customization_type': job.get('customizationType'),
                        'creation_time': str(job.get('creationTime', '')),
                        'end_time': str(job.get('endTime', '')) if job.get('endTime') else None,
                        'last_modified_time': str(job.get('lastModifiedTime', '')),
                        'custom_model_arn': job.get('customModelArn'),
                        'custom_model_name': job.get('customModelName'),
                    },
                    'tags': {}
                })
    except Exception:
        pass

    # Provisioned Model Throughput
    try:
        paginator = bedrock.get_paginator('list_provisioned_model_throughputs')
        for page in paginator.paginate():
            for pmt in page.get('provisionedModelSummaries', []):
                pmt_arn = pmt['provisionedModelArn']
                pmt_name = pmt['provisionedModelName']

                try:
                    # Get provisioned throughput details
                    pmt_response = bedrock.get_provisioned_model_throughput(
                        provisionedModelId=pmt_arn
                    )

                    # Get tags
                    tags = {}
                    try:
                        tag_response = bedrock.list_tags_for_resource(resourceARN=pmt_arn)
                        for tag in tag_response.get('tags', []):
                            tags[tag.get('key', '')] = tag.get('value', '')
                    except Exception:
                        pass

                    resources.append({
                        'service': 'bedrock',
                        'type': 'provisioned-throughput',
                        'id': pmt_name,
                        'arn': pmt_arn,
                        'name': pmt_name,
                        'region': region,
                        'details': {
                            'status': pmt_response.get('status'),
                            'model_arn': pmt_response.get('modelArn'),
                            'desired_model_arn': pmt_response.get('desiredModelArn'),
                            'foundation_model_arn': pmt_response.get('foundationModelArn'),
                            'model_units': pmt_response.get('modelUnits'),
                            'desired_model_units': pmt_response.get('desiredModelUnits'),
                            'commitment_duration': pmt_response.get('commitmentDuration'),
                            'commitment_expiration_time': str(pmt_response.get('commitmentExpirationTime', '')) if pmt_response.get('commitmentExpirationTime') else None,
                            'creation_time': str(pmt_response.get('creationTime', '')),
                            'last_modified_time': str(pmt_response.get('lastModifiedTime', '')),
                        },
                        'tags': tags
                    })
                except Exception:
                    pass
    except Exception:
        pass

    # Guardrails
    try:
        paginator = bedrock.get_paginator('list_guardrails')
        for page in paginator.paginate():
            for guardrail in page.get('guardrails', []):
                guardrail_id = guardrail['id']
                guardrail_arn = guardrail['arn']
                guardrail_name = guardrail['name']

                # Get tags
                tags = {}
                try:
                    tag_response = bedrock.list_tags_for_resource(resourceARN=guardrail_arn)
                    for tag in tag_response.get('tags', []):
                        tags[tag.get('key', '')] = tag.get('value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'bedrock',
                    'type': 'guardrail',
                    'id': guardrail_id,
                    'arn': guardrail_arn,
                    'name': guardrail_name,
                    'region': region,
                    'details': {
                        'status': guardrail.get('status'),
                        'version': guardrail.get('version'),
                        'created_at': str(guardrail.get('createdAt', '')),
                        'updated_at': str(guardrail.get('updatedAt', '')),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # --- Bedrock Agent resources (uses bedrock-agent client) ---
    bedrock_agent = session.client('bedrock-agent', region_name=region)

    # Agents
    try:
        paginator = bedrock_agent.get_paginator('list_agents')
        for page in paginator.paginate():
            for agent_summary in page.get('agentSummaries', []):
                agent_id = agent_summary['agentId']

                try:
                    agent_response = bedrock_agent.get_agent(agentId=agent_id)
                    agent = agent_response['agent']
                    agent_arn = agent['agentArn']
                    agent_name = agent.get('agentName', agent_id)

                    # Get tags
                    tags = {}
                    try:
                        tag_response = bedrock_agent.list_tags_for_resource(resourceArn=agent_arn)
                        tags = tag_response.get('tags', {})
                    except Exception:
                        pass

                    resources.append({
                        'service': 'bedrock',
                        'type': 'agent',
                        'id': agent_id,
                        'arn': agent_arn,
                        'name': agent_name,
                        'region': region,
                        'details': {
                            'status': agent.get('agentStatus'),
                            'foundation_model': agent.get('foundationModel'),
                            'instruction': agent.get('instruction'),
                            'description': agent.get('description'),
                            'agent_version': agent.get('agentVersion'),
                            'idle_session_ttl': agent.get('idleSessionTTLInSeconds'),
                            'agent_resource_role_arn': agent.get('agentResourceRoleArn'),
                            'created_at': str(agent.get('createdAt', '')),
                            'updated_at': str(agent.get('updatedAt', '')),
                            'prepared_at': str(agent.get('preparedAt', '')) if agent.get('preparedAt') else None,
                        },
                        'tags': tags
                    })
                except Exception:
                    pass
    except Exception:
        pass

    # Knowledge Bases
    try:
        paginator = bedrock_agent.get_paginator('list_knowledge_bases')
        for page in paginator.paginate():
            for kb_summary in page.get('knowledgeBaseSummaries', []):
                kb_id = kb_summary['knowledgeBaseId']

                try:
                    kb_response = bedrock_agent.get_knowledge_base(knowledgeBaseId=kb_id)
                    kb = kb_response['knowledgeBase']
                    kb_arn = kb['knowledgeBaseArn']
                    kb_name = kb.get('name', kb_id)

                    # Get tags
                    tags = {}
                    try:
                        tag_response = bedrock_agent.list_tags_for_resource(resourceArn=kb_arn)
                        tags = tag_response.get('tags', {})
                    except Exception:
                        pass

                    # Extract embedding model ARN from knowledge base configuration
                    kb_config = kb.get('knowledgeBaseConfiguration', {})
                    embedding_model_arn = None
                    vector_config = kb_config.get('vectorKnowledgeBaseConfiguration', {})
                    if vector_config:
                        embedding_model_arn = vector_config.get('embeddingModelArn')

                    storage_config = kb.get('storageConfiguration', {})

                    resources.append({
                        'service': 'bedrock',
                        'type': 'knowledge-base',
                        'id': kb_id,
                        'arn': kb_arn,
                        'name': kb_name,
                        'region': region,
                        'details': {
                            'status': kb.get('status'),
                            'description': kb.get('description'),
                            'knowledge_base_type': kb_config.get('type'),
                            'embedding_model_arn': embedding_model_arn,
                            'storage_type': storage_config.get('type'),
                            'role_arn': kb.get('roleArn'),
                            'created_at': str(kb.get('createdAt', '')),
                            'updated_at': str(kb.get('updatedAt', '')),
                        },
                        'tags': tags
                    })

                    # Data Sources for this Knowledge Base
                    try:
                        ds_paginator = bedrock_agent.get_paginator('list_data_sources')
                        for ds_page in ds_paginator.paginate(knowledgeBaseId=kb_id):
                            for ds in ds_page.get('dataSourceSummaries', []):
                                ds_id = ds['dataSourceId']
                                ds_name = ds.get('name', ds_id)

                                resources.append({
                                    'service': 'bedrock',
                                    'type': 'data-source',
                                    'id': ds_id,
                                    'arn': f"arn:aws:bedrock:{region}:{account_id}:knowledge-base/{kb_id}/data-source/{ds_id}",
                                    'name': ds_name,
                                    'region': region,
                                    'details': {
                                        'status': ds.get('status'),
                                        'description': ds.get('description'),
                                        'knowledge_base_id': kb_id,
                                        'updated_at': str(ds.get('updatedAt', '')),
                                    },
                                    'tags': {}
                                })
                    except Exception:
                        pass

                except Exception:
                    pass
    except Exception:
        pass

    return resources



"""
AWS Data Lifecycle Manager (DLM) resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


def collect_dlm_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Data Lifecycle Manager resources: lifecycle policies.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    dlm = session.client('dlm', region_name=region)

    # Lifecycle Policies
    try:
        response = dlm.get_lifecycle_policies()
        for policy_summary in response.get('Policies', []):
            policy_id = policy_summary['PolicyId']

            details = {
                'description': policy_summary.get('Description'),
                'state': policy_summary.get('State'),
                'policy_type': policy_summary.get('PolicyType'),
                'resource_types': policy_summary.get('ResourceTypes', []),
                'default_policy': policy_summary.get('DefaultPolicy'),
            }

            # Get full policy details
            try:
                policy_detail = dlm.get_lifecycle_policy(PolicyId=policy_id)
                policy = policy_detail.get('Policy', {})

                details['execution_role_arn'] = policy.get('ExecutionRoleArn')
                details['date_created'] = str(policy.get('DateCreated', ''))
                details['date_modified'] = str(policy.get('DateModified', ''))
                details['status_message'] = policy.get('StatusMessage')

                # Policy details
                policy_details = policy.get('PolicyDetails', {})
                details['target_tags'] = policy_details.get('TargetTags', [])
                details['schedules_count'] = len(policy_details.get('Schedules', []))
                details['resource_locations'] = policy_details.get('ResourceLocations', [])

                # Get schedule names
                schedules = policy_details.get('Schedules', [])
                details['schedule_names'] = [s.get('Name') for s in schedules]

                tags = policy.get('Tags', {})
            except Exception:
                tags = {}

            resources.append({
                'service': 'dlm',
                'type': 'lifecycle-policy',
                'id': policy_id,
                'arn': f"arn:aws:dlm:{region}:{account_id}:policy/{policy_id}",
                'name': policy_summary.get('Description', policy_id),
                'region': region,
                'details': details,
                'tags': tags
            })
    except Exception:
        pass

    return resources



"""
IoT Core resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional

# IoT Core supported regions
# https://docs.aws.amazon.com/general/latest/gr/iot-core.html
IOT_REGIONS = {
    # US
    'us-east-1', 'us-east-2', 'us-west-1', 'us-west-2',
    # Asia Pacific
    'ap-east-1', 'ap-northeast-1', 'ap-northeast-2', 'ap-south-1',
    'ap-southeast-1', 'ap-southeast-2', 'ap-southeast-5',
    # Canada
    'ca-central-1',
    # Europe
    'eu-central-1', 'eu-north-1', 'eu-west-1', 'eu-west-2', 'eu-west-3', 'eu-south-2',
    # Middle East
    'me-central-1', 'me-south-1',
    # South America
    'sa-east-1',
}


def collect_iot_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect IoT Core resources: things, thing types, thing groups, policies, certificates.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    # Skip unsupported regions to avoid timeouts
    if region and region not in IOT_REGIONS:
        return []

    resources = []
    iot = session.client('iot', region_name=region)

    # IoT Things
    try:
        paginator = iot.get_paginator('list_things')
        for page in paginator.paginate():
            for thing in page.get('things', []):
                thing_name = thing['thingName']
                thing_arn = thing.get('thingArn', f"arn:aws:iot:{region}:{account_id}:thing/{thing_name}")

                # Get tags
                tags = {}
                try:
                    tag_response = iot.list_tags_for_resource(resourceArn=thing_arn)
                    for tag in tag_response.get('tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'iot',
                    'type': 'thing',
                    'id': thing_name,
                    'arn': thing_arn,
                    'name': thing_name,
                    'region': region,
                    'details': {
                        'thing_type_name': thing.get('thingTypeName'),
                        'version': thing.get('version'),
                        'attributes': thing.get('attributes', {}),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # IoT Thing Types
    try:
        paginator = iot.get_paginator('list_thing_types')
        for page in paginator.paginate():
            for thing_type in page.get('thingTypes', []):
                tt_name = thing_type['thingTypeName']
                tt_arn = thing_type.get('thingTypeArn', f"arn:aws:iot:{region}:{account_id}:thingtype/{tt_name}")

                # Get tags
                tags = {}
                try:
                    tag_response = iot.list_tags_for_resource(resourceArn=tt_arn)
                    for tag in tag_response.get('tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'iot',
                    'type': 'thing-type',
                    'id': tt_name,
                    'arn': tt_arn,
                    'name': tt_name,
                    'region': region,
                    'details': {
                        'deprecated': thing_type.get('thingTypeMetadata', {}).get('deprecated'),
                        'deprecation_date': str(thing_type.get('thingTypeMetadata', {}).get('deprecationDate', '')),
                        'creation_date': str(thing_type.get('thingTypeMetadata', {}).get('creationDate', '')),
                        'thing_type_description': thing_type.get('thingTypeProperties', {}).get('thingTypeDescription'),
                        'searchable_attributes': thing_type.get('thingTypeProperties', {}).get('searchableAttributes', []),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # IoT Thing Groups
    try:
        paginator = iot.get_paginator('list_thing_groups')
        for page in paginator.paginate():
            for group in page.get('thingGroups', []):
                group_name = group['groupName']
                group_arn = group.get('groupArn', f"arn:aws:iot:{region}:{account_id}:thinggroup/{group_name}")

                # Get tags
                tags = {}
                try:
                    tag_response = iot.list_tags_for_resource(resourceArn=group_arn)
                    for tag in tag_response.get('tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'iot',
                    'type': 'thing-group',
                    'id': group_name,
                    'arn': group_arn,
                    'name': group_name,
                    'region': region,
                    'details': {},
                    'tags': tags
                })
    except Exception:
        pass

    # IoT Policies
    try:
        paginator = iot.get_paginator('list_policies')
        for page in paginator.paginate():
            for policy in page.get('policies', []):
                policy_name = policy['policyName']
                policy_arn = policy.get('policyArn', f"arn:aws:iot:{region}:{account_id}:policy/{policy_name}")

                # Get tags
                tags = {}
                try:
                    tag_response = iot.list_tags_for_resource(resourceArn=policy_arn)
                    for tag in tag_response.get('tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'iot',
                    'type': 'policy',
                    'id': policy_name,
                    'arn': policy_arn,
                    'name': policy_name,
                    'region': region,
                    'details': {},
                    'tags': tags
                })
    except Exception:
        pass

    # IoT Certificates (active only)
    try:
        paginator = iot.get_paginator('list_certificates')
        for page in paginator.paginate():
            for cert in page.get('certificates', []):
                cert_id = cert['certificateId']
                cert_arn = cert['certificateArn']

                # Skip inactive certificates
                if cert.get('status') != 'ACTIVE':
                    continue

                resources.append({
                    'service': 'iot',
                    'type': 'certificate',
                    'id': cert_id,
                    'arn': cert_arn,
                    'name': cert_id[:12],
                    'region': region,
                    'details': {
                        'status': cert.get('status'),
                        'creation_date': str(cert.get('creationDate', '')),
                        'certificate_mode': cert.get('certificateMode'),
                    },
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
AWS ECR Public resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_ecr_public_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS ECR Public resources: public repositories.

    Note: ECR Public is only available in us-east-1 region.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    # ECR Public is only available in us-east-1
    if region != 'us-east-1':
        return []

    resources = []
    ecr_public = session.client('ecr-public', region_name='us-east-1')

    # Public Repositories
    try:
        paginator = ecr_public.get_paginator('describe_repositories')
        for page in paginator.paginate():
            for repo in page.get('repositories', []):
                repo_name = repo['repositoryName']
                repo_arn = repo.get('repositoryArn', f"arn:aws:ecr-public::{account_id}:repository/{repo_name}")
                repo_uri = repo.get('repositoryUri', '')

                details = {
                    'registry_id': repo.get('registryId'),
                    'repository_uri': repo_uri,
                    'created_at': str(repo.get('createdAt', '')),
                }

                # Get repository catalog data
                try:
                    catalog_response = ecr_public.get_repository_catalog_data(repositoryName=repo_name)
                    catalog = catalog_response.get('catalogData', {})
                    details.update({
                        'description': catalog.get('description'),
                        'about_text': catalog.get('aboutText')[:200] if catalog.get('aboutText') else None,
                        'usage_text': catalog.get('usageText')[:200] if catalog.get('usageText') else None,
                        'architectures': catalog.get('architectures', []),
                        'operating_systems': catalog.get('operatingSystems', []),
                        'logo_url': catalog.get('logoUrl'),
                        'marketplace_certified': catalog.get('marketplaceCertified'),
                    })
                except Exception:
                    pass

                # Get repository policy
                try:
                    policy_response = ecr_public.get_repository_policy(repositoryName=repo_name)
                    details['has_policy'] = True
                except Exception:
                    details['has_policy'] = False

                # Get tags
                tags = {}
                try:
                    tag_response = ecr_public.list_tags_for_resource(resourceArn=repo_arn)
                    for tag in tag_response.get('tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'ecr-public',
                    'type': 'repository',
                    'id': repo_name,
                    'arn': repo_arn,
                    'name': repo_name,
                    'region': 'us-east-1',
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    return resources



"""
AWS Device Farm resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


# Device Farm supported regions (from https://docs.aws.amazon.com/general/latest/gr/devicefarm.html)
DEVICEFARM_REGIONS = {
    'us-west-2',
}


def collect_devicefarm_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Device Farm resources: projects, device pools, and test grid projects.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    # Skip unsupported regions
    if region not in DEVICEFARM_REGIONS:
        return []

    resources = []
    devicefarm = session.client('devicefarm', region_name=region)

    # Projects
    try:
        paginator = devicefarm.get_paginator('list_projects')
        for page in paginator.paginate():
            for project in page.get('projects', []):
                project_arn = project['arn']
                project_name = project.get('name', project_arn.split('/')[-1])

                details = {}

                resources.append({
                    'service': 'devicefarm',
                    'type': 'project',
                    'id': project_arn.split(':')[-1],
                    'arn': project_arn,
                    'name': project_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Test Grid Projects
    try:
        paginator = devicefarm.get_paginator('list_test_grid_projects')
        for page in paginator.paginate():
            for project in page.get('testGridProjects', []):
                project_arn = project['arn']
                project_name = project.get('name', project_arn.split('/')[-1])

                details = {
                    'description': project.get('description'),
                }

                resources.append({
                    'service': 'devicefarm',
                    'type': 'test-grid-project',
                    'id': project_arn.split(':')[-1],
                    'arn': project_arn,
                    'name': project_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # VPCE Configurations
    try:
        paginator = devicefarm.get_paginator('list_vpce_configurations')
        for page in paginator.paginate():
            for vpce in page.get('vpceConfigurations', []):
                vpce_arn = vpce['arn']
                vpce_name = vpce.get('vpceConfigurationName', vpce_arn.split('/')[-1])

                details = {
                    'vpce_service_name': vpce.get('vpceServiceName'),
                    'service_dns_name': vpce.get('serviceDnsName'),
                }

                resources.append({
                    'service': 'devicefarm',
                    'type': 'vpce-configuration',
                    'id': vpce_arn.split(':')[-1],
                    'arn': vpce_arn,
                    'name': vpce_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
OpenSearch Service resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_opensearch_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect OpenSearch Service resources: domains, serverless collections.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []

    # OpenSearch Domains
    opensearch = session.client('opensearch', region_name=region)
    try:
        response = opensearch.list_domain_names()
        domain_names = [d['DomainName'] for d in response.get('DomainNames', [])]

        if domain_names:
            # Describe domains
            desc_response = opensearch.describe_domains(DomainNames=domain_names)
            for domain in desc_response.get('DomainStatusList', []):
                domain_name = domain['DomainName']
                domain_arn = domain['ARN']

                # Get tags
                tags = {}
                try:
                    tag_response = opensearch.list_tags(ARN=domain_arn)
                    for tag in tag_response.get('TagList', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'opensearch',
                    'type': 'domain',
                    'id': domain['DomainId'],
                    'arn': domain_arn,
                    'name': domain_name,
                    'region': region,
                    'details': {
                        'engine_version': domain.get('EngineVersion'),
                        'created': domain.get('Created'),
                        'deleted': domain.get('Deleted'),
                        'processing': domain.get('Processing'),
                        'upgrade_processing': domain.get('UpgradeProcessing'),
                        'endpoint': domain.get('Endpoint'),
                        'endpoints': domain.get('Endpoints'),
                        'instance_type': domain.get('ClusterConfig', {}).get('InstanceType'),
                        'instance_count': domain.get('ClusterConfig', {}).get('InstanceCount'),
                        'dedicated_master_enabled': domain.get('ClusterConfig', {}).get('DedicatedMasterEnabled'),
                        'zone_awareness_enabled': domain.get('ClusterConfig', {}).get('ZoneAwarenessEnabled'),
                        'warm_enabled': domain.get('ClusterConfig', {}).get('WarmEnabled'),
                        'ebs_enabled': domain.get('EBSOptions', {}).get('EBSEnabled'),
                        'ebs_volume_size': domain.get('EBSOptions', {}).get('VolumeSize'),
                        'encryption_at_rest_enabled': domain.get('EncryptionAtRestOptions', {}).get('Enabled'),
                        'node_to_node_encryption_enabled': domain.get('NodeToNodeEncryptionOptions', {}).get('Enabled'),
                        'vpc_id': domain.get('VPCOptions', {}).get('VPCId'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # OpenSearch Serverless Collections
    try:
        oss = session.client('opensearchserverless', region_name=region)
        paginator = oss.get_paginator('list_collections')
        for page in paginator.paginate():
            for collection in page.get('collectionSummaries', []):
                coll_id = collection['id']
                coll_name = collection['name']
                coll_arn = collection['arn']

                # Get tags
                tags = {}
                try:
                    tag_response = oss.list_tags_for_resource(resourceArn=coll_arn)
                    for tag in tag_response.get('tags', []):
                        tags[tag.get('key', '')] = tag.get('value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'opensearch',
                    'type': 'serverless-collection',
                    'id': coll_id,
                    'arn': coll_arn,
                    'name': coll_name,
                    'region': region,
                    'details': {
                        'status': collection.get('status'),
                        'type': collection.get('type'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    return resources



"""
AWS CloudHSM v2 resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


def collect_cloudhsmv2_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS CloudHSM v2 resources: clusters and backups.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    cloudhsm = session.client('cloudhsmv2', region_name=region)

    # Clusters
    try:
        paginator = cloudhsm.get_paginator('describe_clusters')
        for page in paginator.paginate():
            for cluster in page.get('Clusters', []):
                cluster_id = cluster['ClusterId']

                details = {
                    'state': cluster.get('State'),
                    'state_message': cluster.get('StateMessage'),
                    'hsm_type': cluster.get('HsmType'),
                    'vpc_id': cluster.get('VpcId'),
                    'subnet_mapping': cluster.get('SubnetMapping', {}),
                    'security_group': cluster.get('SecurityGroup'),
                    'create_timestamp': str(cluster.get('CreateTimestamp', '')),
                    'backup_policy': cluster.get('BackupPolicy'),
                    'backup_retention_policy': cluster.get('BackupRetentionPolicy', {}),
                    'source_backup_id': cluster.get('SourceBackupId'),
                    'mode': cluster.get('Mode'),
                }

                # HSMs in the cluster
                hsms = cluster.get('Hsms', [])
                details['hsm_count'] = len(hsms)
                details['hsm_ids'] = [h.get('HsmId') for h in hsms]
                details['hsm_states'] = [h.get('State') for h in hsms]

                # Certificates
                certs = cluster.get('Certificates', {})
                if certs:
                    details['cluster_csr'] = bool(certs.get('ClusterCsr'))
                    details['hsm_certificate'] = bool(certs.get('HsmCertificate'))
                    details['aws_hardware_certificate'] = bool(certs.get('AwsHardwareCertificate'))
                    details['manufacturer_hardware_certificate'] = bool(certs.get('ManufacturerHardwareCertificate'))
                    details['cluster_certificate'] = bool(certs.get('ClusterCertificate'))

                # Get tags
                tags = {}
                try:
                    tag_paginator = cloudhsm.get_paginator('list_tags')
                    for tag_page in tag_paginator.paginate(ResourceId=cluster_id):
                        for tag in tag_page.get('TagList', []):
                            tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'cloudhsmv2',
                    'type': 'cluster',
                    'id': cluster_id,
                    'arn': f"arn:aws:cloudhsm:{region}:{account_id}:cluster/{cluster_id}",
                    'name': cluster_id,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Backups
    try:
        paginator = cloudhsm.get_paginator('describe_backups')
        for page in paginator.paginate():
            for backup in page.get('Backups', []):
                backup_id = backup['BackupId']

                details = {
                    'backup_state': backup.get('BackupState'),
                    'cluster_id': backup.get('ClusterId'),
                    'create_timestamp': str(backup.get('CreateTimestamp', '')),
                    'copy_timestamp': str(backup.get('CopyTimestamp', '')),
                    'never_expires': backup.get('NeverExpires'),
                    'source_region': backup.get('SourceRegion'),
                    'source_backup': backup.get('SourceBackup'),
                    'source_cluster': backup.get('SourceCluster'),
                    'delete_timestamp': str(backup.get('DeleteTimestamp', '')),
                    'hsm_type': backup.get('HsmType'),
                    'mode': backup.get('Mode'),
                }

                # Get tags
                tags = {}
                try:
                    tag_paginator = cloudhsm.get_paginator('list_tags')
                    for tag_page in tag_paginator.paginate(ResourceId=backup_id):
                        for tag in tag_page.get('TagList', []):
                            tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'cloudhsmv2',
                    'type': 'backup',
                    'id': backup_id,
                    'arn': f"arn:aws:cloudhsm:{region}:{account_id}:backup/{backup_id}",
                    'name': backup_id,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    return resources



"""
EMR resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_emr_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect EMR resources: clusters, studios.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    emr = session.client('emr', region_name=region)

    # EMR Clusters (active only - STARTING, BOOTSTRAPPING, RUNNING, WAITING)
    try:
        paginator = emr.get_paginator('list_clusters')
        for page in paginator.paginate(ClusterStates=['STARTING', 'BOOTSTRAPPING', 'RUNNING', 'WAITING', 'TERMINATING']):
            for cluster_summary in page.get('Clusters', []):
                cluster_id = cluster_summary['Id']

                try:
                    # Get cluster details
                    cluster_response = emr.describe_cluster(ClusterId=cluster_id)
                    cluster = cluster_response.get('Cluster', {})

                    cluster_arn = cluster.get('ClusterArn', f"arn:aws:elasticmapreduce:{region}:{account_id}:cluster/{cluster_id}")

                    # Get tags
                    tags = {}
                    for tag in cluster.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')

                    resources.append({
                        'service': 'emr',
                        'type': 'cluster',
                        'id': cluster_id,
                        'arn': cluster_arn,
                        'name': cluster.get('Name', cluster_id),
                        'region': region,
                        'details': {
                            'status': cluster.get('Status', {}).get('State'),
                            'status_reason': cluster.get('Status', {}).get('StateChangeReason', {}).get('Message'),
                            'release_label': cluster.get('ReleaseLabel'),
                            'applications': [app.get('Name') for app in cluster.get('Applications', [])],
                            'normalized_instance_hours': cluster.get('NormalizedInstanceHours'),
                            'master_public_dns_name': cluster.get('MasterPublicDnsName'),
                            'log_uri': cluster.get('LogUri'),
                            'auto_terminate': cluster.get('AutoTerminate'),
                            'termination_protected': cluster.get('TerminationProtected'),
                            'visible_to_all_users': cluster.get('VisibleToAllUsers'),
                            'service_role': cluster.get('ServiceRole'),
                            'ec2_instance_attributes': {
                                'key_name': cluster.get('Ec2InstanceAttributes', {}).get('Ec2KeyName'),
                                'subnet_id': cluster.get('Ec2InstanceAttributes', {}).get('Ec2SubnetId'),
                                'availability_zone': cluster.get('Ec2InstanceAttributes', {}).get('Ec2AvailabilityZone'),
                            },
                            'scale_down_behavior': cluster.get('ScaleDownBehavior'),
                            'ebs_root_volume_size': cluster.get('EbsRootVolumeSize'),
                        },
                        'tags': tags
                    })
                except Exception:
                    pass
    except Exception:
        pass

    # EMR Studios
    try:
        paginator = emr.get_paginator('list_studios')
        for page in paginator.paginate():
            for studio_summary in page.get('Studios', []):
                studio_id = studio_summary['StudioId']

                try:
                    # Get studio details
                    studio_response = emr.describe_studio(StudioId=studio_id)
                    studio = studio_response.get('Studio', {})

                    studio_arn = studio.get('StudioArn', f"arn:aws:elasticmapreduce:{region}:{account_id}:studio/{studio_id}")

                    # Get tags
                    tags = {}
                    for tag in studio.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')

                    resources.append({
                        'service': 'emr',
                        'type': 'studio',
                        'id': studio_id,
                        'arn': studio_arn,
                        'name': studio.get('Name', studio_id),
                        'region': region,
                        'details': {
                            'description': studio.get('Description'),
                            'auth_mode': studio.get('AuthMode'),
                            'vpc_id': studio.get('VpcId'),
                            'subnet_ids': studio.get('SubnetIds', []),
                            'url': studio.get('Url'),
                            'default_s3_location': studio.get('DefaultS3Location'),
                            'service_role': studio.get('ServiceRole'),
                            'user_role': studio.get('UserRole'),
                            'workspace_security_group_id': studio.get('WorkspaceSecurityGroupId'),
                            'engine_security_group_id': studio.get('EngineSecurityGroupId'),
                            'creation_time': str(studio.get('CreationTime', '')),
                        },
                        'tags': tags
                    })
                except Exception:
                    pass
    except Exception:
        pass

    # EMR Serverless Applications
    try:
        emr_serverless = session.client('emr-serverless', region_name=region)
        paginator = emr_serverless.get_paginator('list_applications')
        for page in paginator.paginate():
            for app in page.get('applications', []):
                app_id = app['id']
                app_arn = app['arn']

                # Get tags
                tags = {}
                try:
                    tag_response = emr_serverless.list_tags_for_resource(resourceArn=app_arn)
                    tags = tag_response.get('tags', {})
                except Exception:
                    pass

                resources.append({
                    'service': 'emr',
                    'type': 'serverless-application',
                    'id': app_id,
                    'arn': app_arn,
                    'name': app.get('name', app_id),
                    'region': region,
                    'details': {
                        'state': app.get('state'),
                        'state_details': app.get('stateDetails'),
                        'type': app.get('type'),
                        'release_label': app.get('releaseLabel'),
                        'architecture': app.get('architecture'),
                        'created_at': str(app.get('createdAt', '')),
                        'updated_at': str(app.get('updatedAt', '')),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    return resources



"""
Amazon Redshift Serverless resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_redshiftserverless_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect Redshift Serverless resources: namespaces, workgroups, snapshots.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    rsserverless = session.client('redshift-serverless', region_name=region)

    # Namespaces
    try:
        paginator = rsserverless.get_paginator('list_namespaces')
        for page in paginator.paginate():
            for namespace in page.get('namespaces', []):
                namespace_arn = namespace.get('namespaceArn', '')
                namespace_name = namespace.get('namespaceName', namespace_arn.split('/')[-1])

                details = {
                    'namespace_id': namespace.get('namespaceId'),
                    'status': namespace.get('status'),
                    'db_name': namespace.get('dbName'),
                    'admin_username': namespace.get('adminUsername'),
                    'creation_date': str(namespace.get('creationDate', '')) if namespace.get('creationDate') else None,
                }

                resources.append({
                    'service': 'redshift-serverless',
                    'type': 'namespace',
                    'id': namespace.get('namespaceId', namespace_name),
                    'arn': namespace_arn,
                    'name': namespace_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Workgroups
    try:
        paginator = rsserverless.get_paginator('list_workgroups')
        for page in paginator.paginate():
            for workgroup in page.get('workgroups', []):
                workgroup_arn = workgroup.get('workgroupArn', '')
                workgroup_name = workgroup.get('workgroupName', workgroup_arn.split('/')[-1])

                details = {
                    'workgroup_id': workgroup.get('workgroupId'),
                    'namespace_name': workgroup.get('namespaceName'),
                    'status': workgroup.get('status'),
                    'base_capacity': workgroup.get('baseCapacity'),
                    'max_capacity': workgroup.get('maxCapacity'),
                    'enhanced_vpc_routing': workgroup.get('enhancedVpcRouting'),
                    'publicly_accessible': workgroup.get('publiclyAccessible'),
                    'creation_date': str(workgroup.get('creationDate', '')) if workgroup.get('creationDate') else None,
                }

                endpoint = workgroup.get('endpoint', {})
                if endpoint:
                    details['endpoint_address'] = endpoint.get('address')
                    details['endpoint_port'] = endpoint.get('port')

                resources.append({
                    'service': 'redshift-serverless',
                    'type': 'workgroup',
                    'id': workgroup.get('workgroupId', workgroup_name),
                    'arn': workgroup_arn,
                    'name': workgroup_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Snapshots
    try:
        paginator = rsserverless.get_paginator('list_snapshots')
        for page in paginator.paginate():
            for snapshot in page.get('snapshots', []):
                snapshot_arn = snapshot.get('snapshotArn', '')
                snapshot_name = snapshot.get('snapshotName', snapshot_arn.split('/')[-1])

                details = {
                    'namespace_name': snapshot.get('namespaceName'),
                    'namespace_arn': snapshot.get('namespaceArn'),
                    'status': snapshot.get('status'),
                    'snapshot_create_time': str(snapshot.get('snapshotCreateTime', '')) if snapshot.get('snapshotCreateTime') else None,
                    'total_backup_size_in_mega_bytes': snapshot.get('totalBackupSizeInMegaBytes'),
                }

                resources.append({
                    'service': 'redshift-serverless',
                    'type': 'snapshot',
                    'id': snapshot_name,
                    'arn': snapshot_arn,
                    'name': snapshot_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
Systems Manager resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_ssm_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect SSM resources: parameters, documents, maintenance windows, patch baselines.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    ssm = session.client('ssm', region_name=region)

    # SSM Parameters
    try:
        paginator = ssm.get_paginator('describe_parameters')
        for page in paginator.paginate():
            for param in page.get('Parameters', []):
                param_name = param['Name']

                # Get tags
                tags = {}
                try:
                    tag_response = ssm.list_tags_for_resource(
                        ResourceType='Parameter',
                        ResourceId=param_name
                    )
                    for tag in tag_response.get('TagList', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'ssm',
                    'type': 'parameter',
                    'id': param_name,
                    'arn': f"arn:aws:ssm:{region}:{account_id}:parameter{param_name if param_name.startswith('/') else '/' + param_name}",
                    'name': param_name,
                    'region': region,
                    'details': {
                        'type': param.get('Type'),
                        'key_id': param.get('KeyId'),
                        'version': param.get('Version'),
                        'tier': param.get('Tier'),
                        'policies': param.get('Policies', []),
                        'data_type': param.get('DataType'),
                        'last_modified_date': str(param.get('LastModifiedDate', '')),
                        'last_modified_user': param.get('LastModifiedUser'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # SSM Documents (owned by account, not AWS)
    try:
        paginator = ssm.get_paginator('list_documents')
        for page in paginator.paginate(Filters=[{'Key': 'Owner', 'Values': ['Self']}]):
            for doc in page.get('DocumentIdentifiers', []):
                doc_name = doc['Name']

                # Get tags
                tags = {}
                try:
                    tag_response = ssm.list_tags_for_resource(
                        ResourceType='Document',
                        ResourceId=doc_name
                    )
                    for tag in tag_response.get('TagList', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'ssm',
                    'type': 'document',
                    'id': doc_name,
                    'arn': f"arn:aws:ssm:{region}:{account_id}:document/{doc_name}",
                    'name': doc_name,
                    'region': region,
                    'details': {
                        'document_type': doc.get('DocumentType'),
                        'document_format': doc.get('DocumentFormat'),
                        'document_version': doc.get('DocumentVersion'),
                        'platform_types': doc.get('PlatformTypes', []),
                        'schema_version': doc.get('SchemaVersion'),
                        'target_type': doc.get('TargetType'),
                        'created_date': str(doc.get('CreatedDate', '')),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # Maintenance Windows
    try:
        paginator = ssm.get_paginator('describe_maintenance_windows')
        for page in paginator.paginate():
            for mw in page.get('WindowIdentities', []):
                mw_id = mw['WindowId']
                mw_name = mw.get('Name', mw_id)

                # Get tags
                tags = {}
                try:
                    tag_response = ssm.list_tags_for_resource(
                        ResourceType='MaintenanceWindow',
                        ResourceId=mw_id
                    )
                    for tag in tag_response.get('TagList', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'ssm',
                    'type': 'maintenance-window',
                    'id': mw_id,
                    'arn': f"arn:aws:ssm:{region}:{account_id}:maintenancewindow/{mw_id}",
                    'name': mw_name,
                    'region': region,
                    'details': {
                        'description': mw.get('Description'),
                        'enabled': mw.get('Enabled'),
                        'duration': mw.get('Duration'),
                        'cutoff': mw.get('Cutoff'),
                        'schedule': mw.get('Schedule'),
                        'schedule_timezone': mw.get('ScheduleTimezone'),
                        'next_execution_time': str(mw.get('NextExecutionTime', '')),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # Patch Baselines (owned by account)
    try:
        paginator = ssm.get_paginator('describe_patch_baselines')
        for page in paginator.paginate(Filters=[{'Key': 'OWNER', 'Values': ['Self']}]):
            for pb in page.get('BaselineIdentities', []):
                pb_id = pb['BaselineId']
                pb_name = pb.get('BaselineName', pb_id)

                # Get tags
                tags = {}
                try:
                    tag_response = ssm.list_tags_for_resource(
                        ResourceType='PatchBaseline',
                        ResourceId=pb_id
                    )
                    for tag in tag_response.get('TagList', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'ssm',
                    'type': 'patch-baseline',
                    'id': pb_id,
                    'arn': f"arn:aws:ssm:{region}:{account_id}:patchbaseline/{pb_id}",
                    'name': pb_name,
                    'region': region,
                    'details': {
                        'description': pb.get('BaselineDescription'),
                        'operating_system': pb.get('OperatingSystem'),
                        'default_baseline': pb.get('DefaultBaseline'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # Associations
    try:
        paginator = ssm.get_paginator('list_associations')
        for page in paginator.paginate():
            for assoc in page.get('Associations', []):
                assoc_id = assoc.get('AssociationId')
                if not assoc_id:
                    continue

                resources.append({
                    'service': 'ssm',
                    'type': 'association',
                    'id': assoc_id,
                    'arn': f"arn:aws:ssm:{region}:{account_id}:association/{assoc_id}",
                    'name': assoc.get('AssociationName') or assoc.get('Name', assoc_id),
                    'region': region,
                    'details': {
                        'document_name': assoc.get('Name'),
                        'document_version': assoc.get('DocumentVersion'),
                        'association_version': assoc.get('AssociationVersion'),
                        'schedule_expression': assoc.get('ScheduleExpression'),
                        'targets': assoc.get('Targets', []),
                        'last_execution_date': str(assoc.get('LastExecutionDate', '')),
                        'overview': assoc.get('Overview'),
                    },
                    'tags': {}
                })
    except Exception:
        pass

    return resources
